@@ -39,7 +39,7 @@ String FFPTexturing::Type = "FFP_Texturing";
 const String c_ParamTexelEx("texel_");
 
 //-----------------------------------------------------------------------
-FFPTexturing::FFPTexturing() : mIsPointSprite(false)
+FFPTexturing::FFPTexturing() : mIsPointSprite(false), mLateAddBlend(false)
 {   
 }
 
@@ -81,7 +81,7 @@ bool FFPTexturing::resolveUniformParams(TextureUnitParams* textureUnitParams, Pr
     Program* psProgram = programSet->getCpuProgram(GPT_FRAGMENT_PROGRAM);
     
     // Resolve texture sampler parameter.       
-    textureUnitParams->mTextureSampler = psProgram->resolveParameter(textureUnitParams->mTextureSamplerType, textureUnitParams->mTextureSamplerIndex, (uint16)GPV_GLOBAL, "gTextureSampler");
+    textureUnitParams->mTextureSampler = psProgram->resolveParameter(textureUnitParams->mTextureSamplerType, "gTextureSampler", textureUnitParams->mTextureSamplerIndex);
 
     // Resolve texture matrix parameter.
     if (needsTextureMatrix(textureUnitParams->mTextureUnitState))
@@ -111,23 +111,9 @@ bool FFPTexturing::resolveUniformParams(TextureUnitParams* textureUnitParams, Pr
         break;
 
     case TEXCALC_PROJECTIVE_TEXTURE:
-
         mWorldMatrix = vsProgram->resolveParameter(GpuProgramParameters::ACT_WORLD_MATRIX);
-        textureUnitParams->mTextureViewProjImageMatrix = vsProgram->resolveParameter(GCT_MATRIX_4X4, -1, (uint16)GPV_LIGHTS, "gTexViewProjImageMatrix");
-        
-        const TextureUnitState::EffectMap&      effectMap = textureUnitParams->mTextureUnitState->getEffects(); 
-        TextureUnitState::EffectMap::const_iterator effi;
-
-        for (effi = effectMap.begin(); effi != effectMap.end(); ++effi)
-        {
-            if (effi->second.type == TextureUnitState::ET_PROJECTIVE_TEXTURE)
-            {
-                textureUnitParams->mTextureProjector = effi->second.frustum;
-                break;
-            }
-        }
-
-        OgreAssert(textureUnitParams->mTextureProjector, "frustum is NULL");
+        textureUnitParams->mTextureViewProjImageMatrix = vsProgram->resolveParameter(
+            GpuProgramParameters::ACT_TEXTURE_VIEWPROJ_MATRIX, textureUnitParams->mTextureSamplerIndex);
         break;
     }
 
@@ -156,6 +142,10 @@ bool FFPTexturing::resolveFunctionsParams(TextureUnitParams* textureUnitParams,
             if (textureUnitParams->mTextureMatrix.get() == NULL)
                 texCoordContent = Parameter::Content(Parameter::SPC_TEXTURE_COORDINATE0 + textureUnitParams->mTextureUnitState->getTextureCoordSet());
 
+            // assume already resolved
+            if(vsMain->getOutputParameter(texCoordContent, textureUnitParams->mVSInTextureCoordinateType))
+                break;
+
             textureUnitParams->mVSInputTexCoord = vsMain->resolveInputParameter(
                 Parameter::Content(Parameter::SPC_TEXTURE_COORDINATE0 +
                                    textureUnitParams->mTextureUnitState->getTextureCoordSet()),
@@ -268,6 +258,7 @@ bool FFPTexturing::addVSFunctionInvocations(TextureUnitParams* textureUnitParams
     switch (textureUnitParams->mTexCoordCalcMethod)
     {
     case TEXCALC_NONE:
+        if(textureUnitParams->mVSInputTexCoord)
         stage.assign(textureUnitParams->mVSInputTexCoord, textureUnitParams->mVSOutputTexCoord);
         break;
     case TEXCALC_ENVIRONMENT_MAP:
@@ -314,8 +305,7 @@ bool FFPTexturing::addPSFunctionInvocations(TextureUnitParams* textureUnitParams
     
             
     // Add texture sampling code.
-    ParameterPtr texel = psMain->resolveLocalParameter(c_ParamTexelEx + StringConverter::toString(textureUnitParams->mTextureSamplerIndex), GCT_FLOAT4);
-    addPSSampleTexelInvocation(textureUnitParams, psMain, texel, FFP_PS_SAMPLING);
+    ParameterPtr texel = psMain->resolveLocalParameter(GCT_FLOAT4, c_ParamTexelEx + StringConverter::toString(textureUnitParams->mTextureSamplerIndex));
 
     // Build colour argument for source1.
     source1 = getPSArgument(texel, colourBlend.source1, colourBlend.colourArg1, colourBlend.alphaArg1, false);
@@ -323,6 +313,9 @@ bool FFPTexturing::addPSFunctionInvocations(TextureUnitParams* textureUnitParams
     // Build colour argument for source2.
     source2 = getPSArgument(texel, colourBlend.source2, colourBlend.colourArg2, colourBlend.alphaArg2, false);
 
+    if(source1 == texel || source2 == texel || colourBlend.operation == LBX_BLEND_TEXTURE_ALPHA)
+        addPSSampleTexelInvocation(textureUnitParams, psMain, texel, FFP_PS_SAMPLING);
+
     bool needDifferentAlphaBlend = false;
     if (alphaBlend.operation != colourBlend.operation ||
         alphaBlend.source1 != colourBlend.source1 ||
@@ -333,6 +326,11 @@ bool FFPTexturing::addPSFunctionInvocations(TextureUnitParams* textureUnitParams
         alphaBlend.source2 == LBS_MANUAL)
         needDifferentAlphaBlend = true;
 
+    if(mLateAddBlend && colourBlend.operation == LBX_ADD)
+    {
+        groupOrder = FFP_PS_COLOUR_END + 50 + 1; // after PBR lighting
+    }
+
     // Build colours blend
     addPSBlendInvocations(psMain, source1, source2, texel, 
         textureUnitParams->mTextureSamplerIndex,
@@ -348,6 +346,9 @@ bool FFPTexturing::addPSFunctionInvocations(TextureUnitParams* textureUnitParams
         // Build alpha argument for source2.
         source2 = getPSArgument(texel, alphaBlend.source2, alphaBlend.colourArg2, alphaBlend.alphaArg2, true);
 
+        if(source1 == texel || source2 == texel || alphaBlend.operation == LBX_BLEND_TEXTURE_ALPHA)
+            addPSSampleTexelInvocation(textureUnitParams, psMain, texel, FFP_PS_SAMPLING);
+
         // Build alpha blend
         addPSBlendInvocations(psMain, source1, source2, texel,
                               textureUnitParams->mTextureSamplerIndex, alphaBlend, groupOrder,
@@ -551,11 +552,22 @@ bool FFPTexturing::needsTextureMatrix(TextureUnitState* textureUnitState)
 }
 
 
+bool FFPTexturing::setParameter(const String& name, const String& value)
+{
+    if(name == "late_add_blend")
+    {
+        StringConverter::parse(value, mLateAddBlend);
+        return true;
+    }
+    return false;
+}
+
 //-----------------------------------------------------------------------
 void FFPTexturing::copyFrom(const SubRenderState& rhs)
 {
     const FFPTexturing& rhsTexture = static_cast<const FFPTexturing&>(rhs);
 
+    mLateAddBlend = rhsTexture.mLateAddBlend;
     setTextureUnitCount(rhsTexture.getTextureUnitCount());
 
     for (unsigned int i=0; i < rhsTexture.getTextureUnitCount(); ++i)
@@ -569,54 +581,24 @@ bool FFPTexturing::preAddToRenderState(const RenderState* renderState, Pass* src
 {
     mIsPointSprite = srcPass->getPointSpritesEnabled();
 
-    //count the number of texture units we need to process
-    size_t validTexUnits = 0;
-    for (unsigned short i=0; i < srcPass->getNumTextureUnitStates(); ++i)
-    {       
-        if (isProcessingNeeded(srcPass->getTextureUnitState(i)))
-        {
-            ++validTexUnits;
-        }
+    if (auto rs = Root::getSingleton().getRenderSystem())
+    {
+        if (mIsPointSprite && !rs->getCapabilities()->hasCapability(RSC_POINT_SPRITES))
+            return false;
     }
 
-    setTextureUnitCount(validTexUnits);
+    setTextureUnitCount(srcPass->getTextureUnitStates().size());
 
     // Build texture stage sub states.
     for (unsigned short i=0; i < srcPass->getNumTextureUnitStates(); ++i)
     {       
-        TextureUnitState* texUnitState = srcPass->getTextureUnitState(i);                               
-
-        if (isProcessingNeeded(texUnitState))
-        {
-            setTextureUnit(i, texUnitState);    
-        }
+        TextureUnitState* texUnitState = srcPass->getTextureUnitState(i);
+        setTextureUnit(i, texUnitState);
     }   
 
     return true;
 }
 
-//-----------------------------------------------------------------------
-void FFPTexturing::updateGpuProgramsParams(Renderable* rend, const Pass* pass, const AutoParamDataSource* source,
-                                              const LightList* pLightList)
-{
-    for (unsigned int i=0; i < mTextureUnitParamsList.size(); ++i)
-    {
-        TextureUnitParams* curParams = &mTextureUnitParamsList[i];
-
-        if (curParams->mTextureProjector != NULL && curParams->mTextureViewProjImageMatrix.get() != NULL)
-        {                   
-            Matrix4 matTexViewProjImage;
-
-            matTexViewProjImage = 
-                Matrix4::CLIPSPACE2DTOIMAGESPACE * 
-                curParams->mTextureProjector->getProjectionMatrixWithRSDepth() * 
-                curParams->mTextureProjector->getViewMatrix();
-
-            curParams->mTextureViewProjImageMatrix->setGpuParameter(matTexViewProjImage);
-        }
-    }
-}
-
 //-----------------------------------------------------------------------
 void FFPTexturing::setTextureUnitCount(size_t count)
 {
@@ -626,8 +608,7 @@ void FFPTexturing::setTextureUnitCount(size_t count)
     {
         TextureUnitParams& curParams = mTextureUnitParamsList[i];
 
-        curParams.mTextureUnitState             = NULL;         
-        curParams.mTextureProjector             = NULL;               
+        curParams.mTextureUnitState             = NULL;
         curParams.mTextureSamplerIndex          = 0;              
         curParams.mTextureSamplerType           = GCT_SAMPLER2D;        
         curParams.mVSInTextureCoordinateType    = GCT_FLOAT2;   
@@ -639,7 +620,6 @@ void FFPTexturing::setTextureUnitCount(size_t count)
 void FFPTexturing::setTextureUnit(unsigned short index, TextureUnitState* textureUnitState)
 {
     OgreAssert(index < mTextureUnitParamsList.size(), "FFPTexturing unit index out of bounds");
-    OgreAssert(textureUnitState->getBindingType() == TextureUnitState::BT_FRAGMENT, "only fragment shaders supported");
 
     TextureUnitParams& curParams = mTextureUnitParamsList[index];
 
@@ -647,7 +627,7 @@ void FFPTexturing::setTextureUnit(unsigned short index, TextureUnitState* textur
     curParams.mTextureSamplerIndex = index;
     curParams.mTextureUnitState    = textureUnitState;
 
-    bool isGLES2 = Root::getSingletonPtr()->getRenderSystem()->getName().find("OpenGL ES 2") != String::npos;
+    bool isGLES2 = ShaderGenerator::getSingleton().getTargetLanguage() == "glsles";
 
     switch (curParams.mTextureUnitState->getTextureType())
     {
@@ -661,10 +641,6 @@ void FFPTexturing::setTextureUnit(unsigned short index, TextureUnitState* textur
         curParams.mTextureSamplerType = GCT_SAMPLER2D;
         curParams.mVSInTextureCoordinateType = GCT_FLOAT2;
         break;
-    case TEX_TYPE_2D_RECT:
-        curParams.mTextureSamplerType = GCT_SAMPLERRECT;
-        curParams.mVSInTextureCoordinateType = GCT_FLOAT2;
-        break;
     case TEX_TYPE_EXTERNAL_OES:
         curParams.mTextureSamplerType = GCT_SAMPLER_EXTERNAL_OES;
         curParams.mVSInTextureCoordinateType = GCT_FLOAT2;
@@ -686,17 +662,18 @@ void FFPTexturing::setTextureUnit(unsigned short index, TextureUnitState* textur
      curParams.mVSOutTextureCoordinateType = curParams.mVSInTextureCoordinateType;
      curParams.mTexCoordCalcMethod = getTexCalcMethod(curParams.mTextureUnitState);
 
+    // let TexCalcMethod override texture type, as it might be wrong for
+    // content_type shadow & content_type compositor
+    if (curParams.mTexCoordCalcMethod == TEXCALC_ENVIRONMENT_MAP_REFLECTION)
+    {
+        curParams.mVSOutTextureCoordinateType = GCT_FLOAT3;
+        curParams.mTextureSamplerType = GCT_SAMPLERCUBE;
+    }
+
      if (curParams.mTexCoordCalcMethod == TEXCALC_PROJECTIVE_TEXTURE)
          curParams.mVSOutTextureCoordinateType = GCT_FLOAT3;    
 }
 
-//-----------------------------------------------------------------------
-bool FFPTexturing::isProcessingNeeded(TextureUnitState* texUnitState)
-{
-    return texUnitState->getBindingType() == TextureUnitState::BT_FRAGMENT;
-}
-
-
 //-----------------------------------------------------------------------
 const String& FFPTexturingFactory::getType() const
 {
@@ -711,18 +688,23 @@ SubRenderState* FFPTexturingFactory::createInstance(ScriptCompiler* compiler,
     {
         if(prop->values.size() == 1)
         {
-            String modelType;
+            String value;
 
-            if(false == SGScriptTranslator::getString(prop->values.front(), &modelType))
+            if(false == SGScriptTranslator::getString(prop->values.front(), &value))
             {
                 compiler->addError(ScriptCompiler::CE_INVALIDPARAMETERS, prop->file, prop->line);
                 return NULL;
             }
 
-            if (modelType == "ffp")
-            {
-                return createOrRetrieveInstance(translator);
-            }
+            auto inst = createOrRetrieveInstance(translator);
+
+            if (value == "ffp")
+                return inst;
+
+            if (value == "late_add_blend")
+                inst->setParameter(value, "true");
+
+            return inst;
         }       
     }
 
