@@ -54,10 +54,8 @@ namespace RTShader {
         */
         virtual String toString () const
         {
-            const String& lang = ShaderGenerator::getSingleton().getTargetLanguage();
             StringStream str;
-            str << (!lang.empty() && lang[0] == 'g' ? "vec2(" : "float2(");
-            str << std::showpoint << mValue.x << "," << mValue.y << ")";
+            str << "vec2(" << std::showpoint << mValue.x << "," << mValue.y << ")";
             return str.str();
         }
     };
@@ -81,10 +79,8 @@ namespace RTShader {
         */
         virtual String toString () const
         {
-            const String& lang = ShaderGenerator::getSingleton().getTargetLanguage();
             StringStream str;
-            str << (!lang.empty() && lang[0] == 'g' ? "vec3(" : "float3(");
-            str << std::showpoint << mValue.x << "," << mValue.y << "," << mValue.z << ")";
+            str << "vec3(" << std::showpoint << mValue.x << "," << mValue.y << "," << mValue.z << ")";
             return str.str();
         }
     };
@@ -108,10 +104,8 @@ namespace RTShader {
         */
         virtual String toString () const
         {
-            const String& lang = ShaderGenerator::getSingleton().getTargetLanguage();
             StringStream str;
-            str << (!lang.empty() && lang[0] == 'g' ? "vec4(" : "float4(");
-            str << std::showpoint << mValue.x << "," << mValue.y << "," << mValue.z << "," << mValue.w << ")";
+            str << "vec4(" << std::showpoint << mValue.x << "," << mValue.y << "," << mValue.z << "," << mValue.w << ")";
             return str.str();
         }
     };
@@ -176,50 +170,6 @@ Parameter::Parameter(GpuConstantType type, const String& name,
 {
 }
 
-//-----------------------------------------------------------------------
-bool UniformParameter::isFloat() const
-{
-    switch(getType())
-    {
-    case GCT_INT1:
-    case GCT_INT2:
-    case GCT_INT3:
-    case GCT_INT4:
-    case GCT_UINT1:
-    case GCT_UINT2:
-    case GCT_UINT3:
-    case GCT_UINT4:
-    case GCT_SAMPLER1D:
-    case GCT_SAMPLER2D:
-    case GCT_SAMPLER2DARRAY:
-    case GCT_SAMPLER3D:
-    case GCT_SAMPLERCUBE:
-    case GCT_SAMPLER1DSHADOW:
-    case GCT_SAMPLER2DSHADOW:
-        return false;
-    default:
-        return true;
-    };
-}
-
-//-----------------------------------------------------------------------
-bool UniformParameter::isSampler() const
-{
-    switch(getType())
-    {
-    case GCT_SAMPLER1D:
-    case GCT_SAMPLER2D:
-    case GCT_SAMPLER2DARRAY:
-    case GCT_SAMPLER3D:
-    case GCT_SAMPLERCUBE:
-    case GCT_SAMPLER1DSHADOW:
-    case GCT_SAMPLER2DSHADOW:
-        return true;
-    default:
-        return false;
-    };
-}
-
 //-----------------------------------------------------------------------
 UniformParameter::UniformParameter(GpuConstantType type, const String& name, 
                  const Semantic& semantic, int index, 
@@ -262,7 +212,7 @@ static GpuConstantType getGCType(const GpuProgramParameters::AutoConstantDefinit
 }
 
 //-----------------------------------------------------------------------
-UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, Real fAutoConstantData, size_t size)
+UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, float fAutoConstantData, size_t size)
 {
     auto parameterDef = GpuProgramParameters::getAutoConstantDefinition(autoType);
     assert(parameterDef);
@@ -289,7 +239,7 @@ UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoTy
 }
 
 //-----------------------------------------------------------------------
-UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, Real fAutoConstantData, size_t size, GpuConstantType type)
+UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, float fAutoConstantData, size_t size, GpuConstantType type)
 {
     auto parameterDef = GpuProgramParameters::getAutoConstantDefinition(autoType);
     assert(parameterDef);
@@ -316,7 +266,7 @@ UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoTy
 }
 
 //-----------------------------------------------------------------------
-UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, size_t nAutoConstantData, size_t size)
+UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, uint32 nAutoConstantData, size_t size)
 {
     auto parameterDef = GpuProgramParameters::getAutoConstantDefinition(autoType);
     assert(parameterDef);
@@ -339,7 +289,7 @@ UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoTy
 }
 
 //-----------------------------------------------------------------------
-UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, size_t nAutoConstantData, size_t size, GpuConstantType type)
+UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoType, uint32 nAutoConstantData, size_t size, GpuConstantType type)
 {
     auto parameterDef = GpuProgramParameters::getAutoConstantDefinition(autoType);
     assert(parameterDef);
@@ -363,28 +313,25 @@ UniformParameter::UniformParameter(GpuProgramParameters::AutoConstantType autoTy
 
 //-----------------------------------------------------------------------
 void UniformParameter::bind(GpuProgramParametersSharedPtr paramsPtr)
-{   
-    if (paramsPtr.get() != NULL)
-    {
-        // do not throw on failure: some RS optimize unused uniforms away. Also unit tests run without any RS
-        const GpuConstantDefinition* def = paramsPtr->_findNamedConstantDefinition(mBindName.empty() ? mName : mBindName, false);
+{
+    // do not throw on failure: some RS optimize unused uniforms away. Also unit tests run without any RS
+    const GpuConstantDefinition* def = paramsPtr->_findNamedConstantDefinition(mBindName.empty() ? mName : mBindName, false);
 
-        if (def != NULL)
-        {
-            mParamsPtr = paramsPtr.get();
-            mPhysicalIndex = def->physicalIndex;
-            mElementSize = def->elementSize;
-            mVariability = def->variability;
-        }
+    if (def != NULL)
+    {
+        mParamsPtr = paramsPtr.get();
+        mPhysicalIndex = def->physicalIndex;
+        mElementSize = def->elementSize;
+        mVariability = def->variability;
     }
 }
 
 //-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createInPosition(int index)
+ParameterPtr ParameterFactory::createInPosition(int index, Parameter::Content content)
 {
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT4, "iPos_" + StringConverter::toString(index), 
-        Parameter::SPS_POSITION, index, 
-        Parameter::SPC_POSITION_OBJECT_SPACE));
+    return std::make_shared<Parameter>(GCT_FLOAT4, "iPos_" + StringConverter::toString(index),
+                                       Parameter::SPS_POSITION, index,
+                                       content);
 }
 
 //-----------------------------------------------------------------------
@@ -417,7 +364,7 @@ ParameterPtr ParameterFactory::createInWeights(int index)
 ParameterPtr ParameterFactory::createInIndices(int index)
 {
 	return ParameterPtr(OGRE_NEW Parameter(
-		GCT_FLOAT4
+		GCT_UINT4
 	, "iBlendIndices_" + StringConverter::toString(index), 
         Parameter::SPS_BLEND_INDICES, index, 
         Parameter::SPC_BLEND_INDICES));
@@ -485,24 +432,9 @@ ParameterPtr ParameterFactory::createInTexcoord(GpuConstantType type, int index,
     switch (type)
     {
     case GCT_FLOAT1:
-        return createInTexcoord1(index, content);
-        
     case GCT_FLOAT2:
-        return createInTexcoord2(index, content);
-        
     case GCT_FLOAT3:
-        return createInTexcoord3(index, content);
-        
     case GCT_FLOAT4:
-        return createInTexcoord4(index, content);       
-    default:
-    case GCT_SAMPLER1D:
-    case GCT_SAMPLER2D:
-    case GCT_SAMPLER2DARRAY:
-    case GCT_SAMPLER3D:
-    case GCT_SAMPLERCUBE:
-    case GCT_SAMPLER1DSHADOW:
-    case GCT_SAMPLER2DSHADOW:
     case GCT_MATRIX_2X2:
     case GCT_MATRIX_2X3:
     case GCT_MATRIX_2X4:
@@ -520,6 +452,16 @@ ParameterPtr ParameterFactory::createInTexcoord(GpuConstantType type, int index,
     case GCT_UINT2:
     case GCT_UINT3:
     case GCT_UINT4:
+        return std::make_shared<Parameter>(type, StringUtil::format("iTexcoord_%d", index),
+                                           Parameter::SPS_TEXTURE_COORDINATES, index, content);
+    default:
+    case GCT_SAMPLER1D:
+    case GCT_SAMPLER2D:
+    case GCT_SAMPLER2DARRAY:
+    case GCT_SAMPLER3D:
+    case GCT_SAMPLERCUBE:
+    case GCT_SAMPLER1DSHADOW:
+    case GCT_SAMPLER2DSHADOW:
     case GCT_UNKNOWN:
         break;
     }
@@ -533,17 +475,11 @@ ParameterPtr ParameterFactory::createOutTexcoord(GpuConstantType type, int index
     switch (type)
     {
     case GCT_FLOAT1:
-        return createOutTexcoord1(index, content);
-
     case GCT_FLOAT2:
-        return createOutTexcoord2(index, content);
-
     case GCT_FLOAT3:
-        return createOutTexcoord3(index, content);
-
     case GCT_FLOAT4:
-        return createOutTexcoord4(index, content);      
-    
+        return std::make_shared<Parameter>(type, StringUtil::format("oTexcoord_%d", index),
+                                           Parameter::SPS_TEXTURE_COORDINATES, index, content);
     default:
     case GCT_SAMPLER1D:
     case GCT_SAMPLER2D:
@@ -576,70 +512,6 @@ ParameterPtr ParameterFactory::createOutTexcoord(GpuConstantType type, int index
     return ParameterPtr();
 }
 
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createInTexcoord1(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT1, "iTexcoord1_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createOutTexcoord1(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT1, "oTexcoord1_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createInTexcoord2(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT2, "iTexcoord2_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createOutTexcoord2(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT2, "oTexcoord2_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createInTexcoord3(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT3, "iTexcoord3_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createOutTexcoord3(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT3, "oTexcoord3_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createInTexcoord4(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT4, "iTexcoord4_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
-//-----------------------------------------------------------------------
-ParameterPtr ParameterFactory::createOutTexcoord4(int index, Parameter::Content content)
-{
-    return ParameterPtr(OGRE_NEW Parameter(GCT_FLOAT4, "oTexcoord4_" + StringConverter::toString(index), 
-        Parameter::SPS_TEXTURE_COORDINATES, index, 
-        content));
-}
-
 //-----------------------------------------------------------------------
 UniformParameterPtr ParameterFactory::createSampler(GpuConstantType type, int index)
 {
