@@ -64,59 +64,35 @@ ProgramManager::~ProgramManager()
 //-----------------------------------------------------------------------------
 void ProgramManager::releasePrograms(const ProgramSet* programSet)
 {
-    GpuProgramPtr vsProgram(programSet->getGpuProgram(GPT_VERTEX_PROGRAM));
-    GpuProgramPtr psProgram(programSet->getGpuProgram(GPT_FRAGMENT_PROGRAM));
-
-    GpuProgramsMapIterator itVsGpuProgram = !vsProgram ? mVertexShaderMap.end() : mVertexShaderMap.find(vsProgram->getName());
-    GpuProgramsMapIterator itFsGpuProgram = !psProgram ? mFragmentShaderMap.end() : mFragmentShaderMap.find(psProgram->getName());
-
-    if (itVsGpuProgram != mVertexShaderMap.end())
+    for(auto t : {GPT_VERTEX_PROGRAM, GPT_FRAGMENT_PROGRAM})
     {
-        if (itVsGpuProgram->second.use_count() == ResourceGroupManager::RESOURCE_SYSTEM_NUM_REFERENCE_COUNTS + 1)
-        {
-            destroyGpuProgram(itVsGpuProgram->second);
-            mVertexShaderMap.erase(itVsGpuProgram);
-        }
+        const auto& prg = programSet->getGpuProgram(t);
+        if(prg.use_count() > ResourceGroupManager::RESOURCE_SYSTEM_NUM_REFERENCE_COUNTS + 2)
+            continue;
+
+        mShaderList.erase(std::find(mShaderList.begin(), mShaderList.end(), prg));
+        GpuProgramManager::getSingleton().remove(prg);
+    }
+}
+size_t ProgramManager::getShaderCount(GpuProgramType type) const
+{
+    size_t count = 0;
+
+    for(const auto& s : mShaderList)
+    {
+        count += size_t(s->getType() == type);
     }
 
-    if (itFsGpuProgram != mFragmentShaderMap.end())
-    {
-        if (itFsGpuProgram->second.use_count() == ResourceGroupManager::RESOURCE_SYSTEM_NUM_REFERENCE_COUNTS + 1)
-        {
-            destroyGpuProgram(itFsGpuProgram->second);
-            mFragmentShaderMap.erase(itFsGpuProgram);
-        }
-    }
+    return count;
 }
 //-----------------------------------------------------------------------------
 void ProgramManager::flushGpuProgramsCache()
 {
-    flushGpuProgramsCache(mVertexShaderMap);
-    flushGpuProgramsCache(mFragmentShaderMap);
-}
-
-size_t ProgramManager::getShaderCount(GpuProgramType type) const
-{
-    switch(type)
+    for(auto& s : mShaderList)
     {
-    case GPT_VERTEX_PROGRAM:
-        return mVertexShaderMap.size();
-    case GPT_FRAGMENT_PROGRAM:
-        return mFragmentShaderMap.size();
-    default:
-        return 0;
-    }
-}
-//-----------------------------------------------------------------------------
-void ProgramManager::flushGpuProgramsCache(GpuProgramsMap& gpuProgramsMap)
-{
-    while (gpuProgramsMap.size() > 0)
-    {
-        GpuProgramsMapIterator it = gpuProgramsMap.begin();
-
-        destroyGpuProgram(it->second);
-        gpuProgramsMap.erase(it);
+        GpuProgramManager::getSingleton().remove(s);
     }
+    mShaderList.clear();
 }
 //-----------------------------------------------------------------------------
 void ProgramManager::createDefaultProgramProcessors()
@@ -223,18 +199,16 @@ GpuProgramPtr ProgramManager::createGpuProgram(Program* shaderProgram,
     }
 
     // Try to get program by name.
-    HighLevelGpuProgramPtr pGpuProgram =
-        HighLevelGpuProgramManager::getSingleton().getByName(
-            programName, ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
+    auto pGpuProgram = GpuProgramManager::getSingleton().getByName(programName, RGN_INTERNAL);
 
     if(pGpuProgram) {
-        return static_pointer_cast<GpuProgram>(pGpuProgram);
+        return pGpuProgram;
     }
 
     // Case the program doesn't exist yet.
     // Create new GPU program.
-    pGpuProgram = HighLevelGpuProgramManager::getSingleton().createProgram(programName,
-        ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, language, shaderProgram->getType());
+    pGpuProgram =
+        GpuProgramManager::getSingleton().createProgram(programName, RGN_INTERNAL, language, shaderProgram->getType());
 
     // Case cache directory specified -> create program from file.
     if (!cachePath.empty())
@@ -285,17 +259,10 @@ GpuProgramPtr ProgramManager::createGpuProgram(Program* shaderProgram,
 
     pGpuProgram->load();
 
-    // Add the created GPU program to local cache.
-    if (pGpuProgram->getType() == GPT_VERTEX_PROGRAM)
-    {
-        mVertexShaderMap[programName] = pGpuProgram;
-    }
-    else if (pGpuProgram->getType() == GPT_FRAGMENT_PROGRAM)
-    {
-        mFragmentShaderMap[programName] = pGpuProgram;
-    }
+    // Add the created GPU program to local index
+    mShaderList.push_back(pGpuProgram);
     
-    return static_pointer_cast<GpuProgram>(pGpuProgram);
+    return pGpuProgram;
 }
 
 
@@ -336,12 +303,6 @@ void ProgramManager::removeProgramProcessor(const String& lang)
 
 }
 
-//-----------------------------------------------------------------------------
-void ProgramManager::destroyGpuProgram(GpuProgramPtr& gpuProgram)
-{       
-    GpuProgramManager::getSingleton().remove(gpuProgram);
-}
-
 //-----------------------------------------------------------------------
 void ProgramManager::synchronizePixelnToBeVertexOut( ProgramSet* programSet )
 {
