@@ -51,7 +51,7 @@ namespace Ogre
     typedef std::pair<bool, Real> RayTestResult;
 
     /** Wrapper class which indicates a given angle value is in Radians.
-    @remarks
+
         Radian values are interchangeable with Degree values, and conversions
         will be done automatically between them.
     */
@@ -103,7 +103,7 @@ namespace Ogre
     };
 
     /** Wrapper class which indicates a given angle value is in Degrees.
-    @remarks
+
         Degree values are interchangeable with Radian values, and conversions
         will be done automatically between them.
     */
@@ -156,7 +156,7 @@ namespace Ogre
 
     /** Wrapper class which identifies a value as the currently default angle 
         type, as defined by Math::setAngleUnit.
-    @remarks
+
         Angle values will be automatically converted between radians and degrees,
         as appropriate.
     */
@@ -192,7 +192,7 @@ namespace Ogre
     }
 
     /** Class to provide access to common mathematical functions.
-        @remarks
+
             Most of the maths functions are aliased versions of the C runtime
             library functions. They are aliased here to provide future
             optimisation opportunities, either from faster RTLs or custom
@@ -358,9 +358,9 @@ namespace Ogre
         static inline Real Log (Real fValue) { return std::log(fValue); }
 
         /// Stored value of log(2) for frequent use
-        static const Real LOG2;
+        static constexpr Real LOG2 = static_cast<Real> (0.69314718055994530942);
 
-        static inline Real Log2 (Real fValue) { return std::log(fValue)/LOG2; }
+        static inline Real Log2 (Real fValue) { return std::log2(fValue); }
 
         static inline Real LogN (Real base, Real fValue) { return std::log(fValue)/std::log(base); }
 
@@ -533,7 +533,7 @@ namespace Ogre
 
        /** Checks whether a given point is inside a triangle, in a
             2-dimensional (Cartesian) space.
-            @remarks
+
                 The vertices of the triangle must be given in either
                 trigonometrical (anticlockwise) or inverse trigonometrical
                 (clockwise) order.
@@ -556,7 +556,7 @@ namespace Ogre
             const Vector2& b, const Vector2& c);
 
        /** Checks whether a given 3D point is inside a triangle.
-       @remarks
+
             The vertices of the triangle must be given in either
             trigonometrical (anticlockwise) or inverse trigonometrical
             (clockwise) order, and the point must be guaranteed to be in the
@@ -738,13 +738,13 @@ namespace Ogre
         static Real boundingRadiusFromAABBCentered(const AxisAlignedBox &aabb);
 
 
-        static const Real POS_INFINITY;
-        static const Real NEG_INFINITY;
-        static const Real PI;
-        static const Real TWO_PI;
-        static const Real HALF_PI;
-        static const float fDeg2Rad;
-        static const float fRad2Deg;
+        static constexpr Real POS_INFINITY = std::numeric_limits<Real>::infinity();
+        static constexpr Real NEG_INFINITY = -std::numeric_limits<Real>::infinity();
+        static constexpr Real PI = static_cast<Real> (3.14159265358979323846);
+        static constexpr Real TWO_PI = Real( 2.0 * PI );
+        static constexpr Real HALF_PI = Real( 0.5 * PI );
+        static constexpr float fDeg2Rad = PI / Real(180.0);
+        static constexpr float fRad2Deg = Real(180.0) / PI;
 
     };
 
