@@ -106,7 +106,7 @@ namespace Ogre {
     {
     public:
         /** Definition of a functor for calculating the hashcode of a Pass.
-            @remarks
+
             The hashcode of a Pass is used to sort Passes for rendering, in order
             to reduce the number of render state changes. Each Pass represents a
             single unique set of states, but by ordering them, state changes can
@@ -277,7 +277,7 @@ namespace Ogre {
         /// Gets the index of this Pass in the parent Technique
         unsigned short getIndex(void) const { return mIndex; }
         /** Set the name of the pass
-           @remarks
+
            The name of the pass is optional.  Its useful in material scripts where a material could inherit
            from another material and only want to modify a particular pass.
         */
@@ -443,7 +443,7 @@ namespace Ogre {
         /// @name Point Sprites
         /// @{
         /** Gets the point size of the pass.
-            @remarks
+
             This property determines what point size is used to render a point
             list.
         */
@@ -532,7 +532,7 @@ namespace Ogre {
         /// @name Texture Units
         /// @{
         /** Inserts a new TextureUnitState object into the Pass.
-            @remarks
+
             This unit is is added on top of all previous units.
             @param textureName
             The basic name of the texture e.g. brickwall.jpg, stonefloor.png
@@ -581,7 +581,7 @@ namespace Ogre {
         const TextureUnitStates& getTextureUnitStates() const { return mTextureUnitStates; }
 
         /** Removes the indexed texture unit state from this pass.
-            @remarks
+
             Note that removing a texture which is not the topmost will have a larger performance impact.
         */
         void removeTextureUnitState(unsigned short index);
@@ -594,7 +594,7 @@ namespace Ogre {
         size_t getNumTextureUnitStates(void) const { return mTextureUnitStates.size(); }
 
         /** Gets the 'nth' texture which references the given content type.
-            @remarks
+
             If the 'nth' texture unit which references the content type doesn't
             exist, then this method returns an arbitrary high-value outside the
             valid range to index texture units.
@@ -822,7 +822,7 @@ namespace Ogre {
         float getDepthBiasSlopeScale(void) const { return mDepthBiasSlopeScale; }
         /** Sets a factor which derives an additional depth bias from the number
             of times a pass is iterated.
-            @remarks
+
             The Final depth bias will be the constant depth bias as set through
             setDepthBias, plus this value times the iteration number.
         */
@@ -914,7 +914,7 @@ namespace Ogre {
         /// @name Fogging
         /// @{
         /** Sets the fogging mode applied to this pass.
-            @remarks
+
             Fogging is an effect that is applied as polys are rendered. Sometimes, you want
             fog to be applied to an entire scene. Other times, you want it to be applied to a few
             polygons only. This pass-level specification of fog parameters lets you easily manage
@@ -984,7 +984,7 @@ namespace Ogre {
         /// @name Alpha Rejection
         /// @{
         /** Sets the way the pass will have use alpha to totally reject pixels from the pipeline.
-            @remarks
+
             The default is CMPF_ALWAYS_PASS i.e. alpha is not used to reject pixels.
             @param func The comparison which must pass for the pixel to be written.
             @param value 1 byte value against which alpha values will be tested(0-255)
@@ -1028,7 +1028,7 @@ namespace Ogre {
         /** Sets whether or not transparent sorting is enabled.
             @param enabled
             If false depth sorting of this material will be disabled.
-            @remarks
+
             By default all transparent materials are sorted such that renderables furthest
             away from the camera are rendered first. This is usually the desired behaviour
             but in certain cases this depth sorting may be unnecessary and undesirable. If
@@ -1047,7 +1047,7 @@ namespace Ogre {
             @param enabled
             If true depth sorting of this material will be depend only on the value of
             getTransparentSortingEnabled().
-            @remarks
+
             By default even if transparent sorting is enabled, depth sorting will only be
             performed when the material is transparent and depth write/check are disabled.
             This function disables these extra conditions.
@@ -1061,7 +1061,7 @@ namespace Ogre {
         /// @name Light Iteration
         /// @{
         /** Sets the maximum number of lights to be used by this pass.
-            @remarks
+
             During rendering, if lighting is enabled (or if the pass uses an automatic
             program parameter based on a light) the engine will request the nearest lights
             to the object being rendered in order to work out which ones to use. This
@@ -1093,7 +1093,7 @@ namespace Ogre {
 
         /** Sets whether or not this pass should iterate per light or number of
             lights which can affect the object being rendered.
-            @remarks
+
             The default behaviour for a pass (when this option is 'false'), is
             for a pass to be rendered only once (or the number of times set in
             setPassIterationCount), with all the lights which could
@@ -1146,7 +1146,7 @@ namespace Ogre {
 
         /** If light iteration is enabled, determine the number of lights per
             iteration.
-            @remarks
+
             The default for this setting is 1, so if you enable light iteration
             (Pass::setIteratePerLight), the pass is rendered once per light. If
             you set this value higher, the passes will occur once per 'n' lights.
@@ -1206,7 +1206,7 @@ namespace Ogre {
         bool hasGpuProgram(GpuProgramType programType) const;
 
         /** Sets the details of the program to use.
-            @remarks
+
             Only applicable to programmable passes, this sets the details of
             the program to use in this pass. The program will not be
             loaded until the parent Material is loaded.
@@ -1252,7 +1252,7 @@ namespace Ogre {
         const String& getComputeProgramName(void) const { return getGpuProgramName(GPT_COMPUTE_PROGRAM); }
 
         /** Sets the Gpu program parameters.
-            @remarks
+
             Only applicable to programmable passes, and this particular call is
             designed for low-level programs; use the named parameter methods
             for setting high-level program parameters.
@@ -1289,7 +1289,7 @@ namespace Ogre {
 
         /** Splits this Pass to one which can be handled in the number of
             texture units specified.
-            @remarks
+
             Only works on non-programmable passes, programmable passes cannot be
             split, it's up to the author to ensure that there is a fallback Technique
             for less capable cards.
@@ -1315,7 +1315,7 @@ namespace Ogre {
         bool isLoaded(void) const;
 
         /** Gets the 'hash' of this pass, ie a precomputed number to use for sorting
-            @remarks
+
             This hash is used to sort passes, and for this reason the pass is hashed
             using firstly its index (so that all passes are rendered in order), then
             by the textures which it's TextureUnitState instances are using.
@@ -1324,7 +1324,7 @@ namespace Ogre {
         /// Mark the hash as dirty
         void _dirtyHash(void);
         /** Internal method for recalculating the hash.
-            @remarks
+
             Do not call this unless you are sure the old hash is not still being
             used by anything. If in doubt, call _dirtyHash if you want to force
             recalculation of the has next time.
@@ -1366,7 +1366,7 @@ namespace Ogre {
         { return msPassGraveyard; }
         /** Static method to reset the list of passes which need their hash
             values recalculated.
-            @remarks
+
             For performance, the dirty list is not updated progressively as
             the hashes are recalculated, instead we expect the processor of the
             dirty hash list to clear the list when they are done.
@@ -1385,7 +1385,7 @@ namespace Ogre {
 
         /** set the number of iterations that this pass
             should perform when doing fast multi pass operation.
-            @remarks
+
             Only applicable for programmable passes.
             @param count number of iterations to perform fast multi pass operations.
             A value greater than 1 will cause the pass to be executed count number of
@@ -1505,7 +1505,7 @@ namespace Ogre {
             MIN_GPU_PROGRAM_CHANGE
         };
         /** Sets one of the default hash functions to be used.
-            @remarks
+
             You absolutely must not change the hash function whilst any Pass instances
             exist in the render queue. The only time you can do this is either
             before you render anything, or directly after you manuall call
@@ -1519,7 +1519,7 @@ namespace Ogre {
         static void setHashFunction(BuiltinHashFunction builtin);
 
         /** Set the hash function used for all passes.
-            @remarks
+
             You absolutely must not change the hash function whilst any Pass instances
             exist in the render queue. The only time you can do this is either
             before you render anything, or directly after you manuall call
@@ -1539,16 +1539,10 @@ namespace Ogre {
          */
         static HashFunc* getBuiltinHashFunction(BuiltinHashFunction builtin);
 
-        /** Return an instance of user objects binding associated with this class.
-            You can use it to associate one or more custom objects with this class instance.
-            @see UserObjectBindings::setUserAny.
-        */
+        /// @copydoc UserObjectBindings
         UserObjectBindings&     getUserObjectBindings() { return mUserObjectBindings; }
 
-        /** Return an instance of user objects binding associated with this class.
-            You can use it to associate one or more custom objects with this class instance.
-            @see UserObjectBindings::setUserAny.
-        */
+        /// @overload
         const UserObjectBindings& getUserObjectBindings() const { return mUserObjectBindings; }
      private:
         std::unique_ptr<GpuProgramUsage>& getProgramUsage(GpuProgramType programType);
@@ -1556,7 +1550,7 @@ namespace Ogre {
     };
 
     /** Struct recording a pass which can be used for a specific illumination stage.
-        @remarks
+
         This structure is used to record categorised passes which fit into a
         number of distinct illumination phases - ambient, diffuse / specular
         (per-light) and decal (post-lighting texturing).
