@@ -38,13 +38,9 @@ THE SOFTWARE.
 namespace Ogre {
     const size_t BillboardChain::SEGMENT_EMPTY = std::numeric_limits<size_t>::max();
     //-----------------------------------------------------------------------
-    BillboardChain::Element::Element()
-    {
-    }
-    //-----------------------------------------------------------------------
     BillboardChain::Element::Element(const Vector3 &_position,
-        Real _width,
-        Real _texCoord,
+        float _width,
+        float _texCoord,
         const ColourValue &_colour,
         const Quaternion &_orientation) :
     position(_position),
@@ -434,11 +430,8 @@ namespace Ogre {
     void BillboardChain::updateVertexBuffer(Camera* cam)
     {
         setupBuffers();
-        
-        // The contents of the vertex buffer are correct if they are not dirty
-        // and the camera used to build the vertex buffer is still the current 
-        // camera.
-        if (!mVertexContentDirty && mVertexCameraUsed == cam)
+
+        if (!mVertexContentDirty && false) // TODO: mAutoUpdate flag like BillboardSet
             return;
 
         HardwareVertexBufferSharedPtr pBuffer =
