@@ -4,36 +4,32 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-
 #include "OgreOptimisedUtil.h"
-#include "OgrePlatformInformation.h"
 
-#if __OGRE_HAVE_SSE
 
-#include "OgreMatrix4.h"
+#if __OGRE_HAVE_SSE || __OGRE_HAVE_NEON
 
 // Should keep this includes at latest to avoid potential "xmmintrin.h" included by
 // other header file on some platform for some reason.
@@ -77,7 +73,7 @@ namespace Ogre {
     class _OgrePrivate OptimisedUtilSSE : public OptimisedUtil
     {
     protected:
-        /// Does we prefer use general SSE version for position/normals shared buffers?
+        /// Do we prefer to use a general SSE version for position/normal shared buffers?
         bool mPreferGeneralVersionForSharedBuffers;
 
     public:
@@ -85,11 +81,11 @@ namespace Ogre {
         OptimisedUtilSSE(void);
 
         /// @copydoc OptimisedUtil::softwareVertexSkinning
-        virtual void softwareVertexSkinning(
+        virtual void __OGRE_SIMD_ALIGN_ATTRIBUTE softwareVertexSkinning(
             const float *srcPosPtr, float *destPosPtr,
             const float *srcNormPtr, float *destNormPtr,
             const float *blendWeightPtr, const unsigned char* blendIndexPtr,
-            const Matrix4* const* blendMatrices,
+            const Affine3* const* blendMatrices,
             size_t srcPosStride, size_t destPosStride,
             size_t srcNormStride, size_t destNormStride,
             size_t blendWeightStride, size_t blendIndexStride,
@@ -97,35 +93,37 @@ namespace Ogre {
             size_t numVertices);
 
         /// @copydoc OptimisedUtil::softwareVertexMorph
-        virtual void softwareVertexMorph(
+        virtual void __OGRE_SIMD_ALIGN_ATTRIBUTE softwareVertexMorph(
             Real t,
             const float *srcPos1, const float *srcPos2,
             float *dstPos,
-            size_t numVertices);
+            size_t pos1VSize, size_t pos2VSize, size_t dstVSize, 
+            size_t numVertices,
+            bool morphNormals);
 
         /// @copydoc OptimisedUtil::concatenateAffineMatrices
-        virtual void concatenateAffineMatrices(
-            const Matrix4& baseMatrix,
-            const Matrix4* srcMatrices,
-            Matrix4* dstMatrices,
+        virtual void __OGRE_SIMD_ALIGN_ATTRIBUTE concatenateAffineMatrices(
+            const Affine3& baseMatrix,
+            const Affine3* srcMatrices,
+            Affine3* dstMatrices,
             size_t numMatrices);
 
         /// @copydoc OptimisedUtil::calculateFaceNormals
-        virtual void calculateFaceNormals(
+        virtual void __OGRE_SIMD_ALIGN_ATTRIBUTE calculateFaceNormals(
             const float *positions,
             const EdgeData::Triangle *triangles,
             Vector4 *faceNormals,
             size_t numTriangles);
 
         /// @copydoc OptimisedUtil::calculateLightFacing
-        virtual void calculateLightFacing(
+        virtual void __OGRE_SIMD_ALIGN_ATTRIBUTE calculateLightFacing(
             const Vector4& lightPos,
             const Vector4* faceNormals,
             char* lightFacings,
             size_t numFaces);
 
         /// @copydoc OptimisedUtil::extrudeVertices
-        virtual void extrudeVertices(
+        virtual void __OGRE_SIMD_ALIGN_ATTRIBUTE extrudeVertices(
             const Vector4& lightPos,
             Real extrudeDist,
             const float* srcPositions,
@@ -135,7 +133,7 @@ namespace Ogre {
 
 #if defined(__OGRE_SIMD_ALIGN_STACK)
     /** Stack-align implementation of OptimisedUtil.
-    @remarks
+
         User code compiled by icc and gcc might not align stack
         properly, we need ensure stack align to a 16-bytes boundary
         when execute SSE function.
@@ -164,7 +162,7 @@ namespace Ogre {
             const float *srcPosPtr, float *destPosPtr,
             const float *srcNormPtr, float *destNormPtr,
             const float *blendWeightPtr, const unsigned char* blendIndexPtr,
-            const Matrix4* const* blendMatrices,
+            const Affine3* const* blendMatrices,
             size_t srcPosStride, size_t destPosStride,
             size_t srcNormStride, size_t destNormStride,
             size_t blendWeightStride, size_t blendIndexStride,
@@ -190,7 +188,9 @@ namespace Ogre {
             Real t,
             const float *srcPos1, const float *srcPos2,
             float *dstPos,
-            size_t numVertices)
+            size_t pos1VSize, size_t pos2VSize, size_t dstVSize, 
+            size_t numVertices,
+            bool morphNormals)
         {
             __OGRE_SIMD_ALIGN_STACK();
 
@@ -198,14 +198,16 @@ namespace Ogre {
                 t,
                 srcPos1, srcPos2,
                 dstPos,
-                numVertices);
+                pos1VSize, pos2VSize, dstVSize, 
+                numVertices,
+                morphNormals);
         }
 
         /// @copydoc OptimisedUtil::concatenateAffineMatrices
         virtual void concatenateAffineMatrices(
-            const Matrix4& baseMatrix,
-            const Matrix4* srcMatrices,
-            Matrix4* dstMatrices,
+            const Affine3& baseMatrix,
+            const Affine3* srcMatrices,
+            Affine3* dstMatrices,
             size_t numMatrices)
         {
             __OGRE_SIMD_ALIGN_STACK();
@@ -386,7 +388,7 @@ namespace Ogre {
     {                                                                           \
         /* Important Note: If reuse pMatrixXXX frequently, M$ VC7.1 will */     \
         /* generate wrong code here!!!                                   */     \
-        const Matrix4* pMatrix0, *pMatrix1, *pMatrix2, *pMatrix3;               \
+        const Affine3* pMatrix0, *pMatrix1, *pMatrix2, *pMatrix3;               \
         __m128 weight, weights;                                                 \
                                                                                 \
         switch (numWeightsPerVertex)                                            \
@@ -440,7 +442,7 @@ namespace Ogre {
     {                                                                           \
         /* Important Note: If reuse pMatrixXXX frequently, M$ VC7.1 will */     \
         /* generate wrong code here!!!                                   */     \
-        const Matrix4* pMatrix0, *pMatrix1, *pMatrix2, *pMatrix3;               \
+        const Affine3* pMatrix0, *pMatrix1, *pMatrix2, *pMatrix3;               \
         __m128 weight, weights;                                                 \
                                                                                 \
         switch (numWeightsPerVertex)                                            \
@@ -509,14 +511,13 @@ namespace Ogre {
     }
 
 
-
     //---------------------------------------------------------------------
     // General SSE version skinning positions, and optional skinning normals.
     static void softwareVertexSkinning_SSE_General(
         const float *pSrcPos, float *pDestPos,
         const float *pSrcNorm, float *pDestNorm,
         const float *pBlendWeight, const unsigned char* pBlendIndex,
-        const Matrix4* const* blendMatrices,
+        const Affine3* const* blendMatrices,
         size_t srcPosStride, size_t destPosStride,
         size_t srcNormStride, size_t destNormStride,
         size_t blendWeightStride, size_t blendIndexStride,
@@ -608,7 +609,7 @@ namespace Ogre {
         static void apply(
             const float* pSrc, float* pDest,
             const float* pBlendWeight, const unsigned char* pBlendIndex,
-            const Matrix4* const* blendMatrices,
+            const Affine3* const* blendMatrices,
             size_t blendWeightStride, size_t blendIndexStride,
             size_t numWeightsPerVertex,
             size_t numIterations)
@@ -726,17 +727,17 @@ namespace Ogre {
             }
         }
     };
-    static FORCEINLINE void softwareVertexSkinning_SSE_PosNorm_Shared_Packed(
+    static OGRE_FORCE_INLINE void softwareVertexSkinning_SSE_PosNorm_Shared_Packed(
             const float* pSrcPos, float* pDestPos,
             const float* pBlendWeight, const unsigned char* pBlendIndex,
-            const Matrix4* const* blendMatrices,
+            const Affine3* const* blendMatrices,
             size_t blendWeightStride, size_t blendIndexStride,
             size_t numWeightsPerVertex,
             size_t numIterations)
     {
         // pSrcPos might can't align to 16 bytes because 8 bytes alignment shift per-vertex
 
-        // Instantiating two version only, since other alignement combination not that important.
+        // Instantiating two version only, since other alignment combinations are not that important.
         if (_isAlignedForSSE(pSrcPos) && _isAlignedForSSE(pDestPos))
         {
             SoftwareVertexSkinning_SSE_PosNorm_Shared_Packed<true, true>::apply(
@@ -768,7 +769,7 @@ namespace Ogre {
             const float* pSrcPos, float* pDestPos,
             const float* pSrcNorm, float* pDestNorm,
             const float* pBlendWeight, const unsigned char* pBlendIndex,
-            const Matrix4* const* blendMatrices,
+            const Affine3* const* blendMatrices,
             size_t blendWeightStride, size_t blendIndexStride,
             size_t numWeightsPerVertex,
             size_t numIterations)
@@ -881,18 +882,18 @@ namespace Ogre {
             }
         }
     };
-    static FORCEINLINE void softwareVertexSkinning_SSE_PosNorm_Separated_Packed(
+    static OGRE_FORCE_INLINE void softwareVertexSkinning_SSE_PosNorm_Separated_Packed(
         const float* pSrcPos, float* pDestPos,
         const float* pSrcNorm, float* pDestNorm,
         const float* pBlendWeight, const unsigned char* pBlendIndex,
-        const Matrix4* const* blendMatrices,
+        const Affine3* const* blendMatrices,
         size_t blendWeightStride, size_t blendIndexStride,
         size_t numWeightsPerVertex,
         size_t numIterations)
     {
         assert(_isAlignedForSSE(pSrcPos));
 
-        // Instantiating two version only, since other alignement combination not that important.
+        // Instantiating two version only, since other alignment combination not that important.
         if (_isAlignedForSSE(pSrcNorm) && _isAlignedForSSE(pDestPos) && _isAlignedForSSE(pDestNorm))
         {
             SoftwareVertexSkinning_SSE_PosNorm_Separated_Packed<true, true, true, true>::apply(
@@ -925,7 +926,7 @@ namespace Ogre {
         static void apply(
             const float* pSrcPos, float* pDestPos,
             const float* pBlendWeight, const unsigned char* pBlendIndex,
-            const Matrix4* const* blendMatrices,
+            const Affine3* const* blendMatrices,
             size_t blendWeightStride, size_t blendIndexStride,
             size_t numWeightsPerVertex,
             size_t numIterations)
@@ -1000,17 +1001,17 @@ namespace Ogre {
             }
         }
     };
-    static FORCEINLINE void softwareVertexSkinning_SSE_PosOnly_Packed(
+    static OGRE_FORCE_INLINE void softwareVertexSkinning_SSE_PosOnly_Packed(
         const float* pSrcPos, float* pDestPos,
         const float* pBlendWeight, const unsigned char* pBlendIndex,
-        const Matrix4* const* blendMatrices,
+        const Affine3* const* blendMatrices,
         size_t blendWeightStride, size_t blendIndexStride,
         size_t numWeightsPerVertex,
         size_t numIterations)
     {
         assert(_isAlignedForSSE(pSrcPos));
 
-        // Instantiating two version only, since other alignement combination not that important.
+        // Instantiating two version only, since other alignment combination not that important.
         if (_isAlignedForSSE(pDestPos))
         {
             SoftwareVertexSkinning_SSE_PosOnly_Packed<true, true>::apply(
@@ -1043,8 +1044,8 @@ namespace Ogre {
         // run out of usable CPU registers, or L1/L2 cache related problem, causing
         // slight performance loss than general version.
         //
-
-		if (PlatformInformation::getCpuIdentifier().find("AuthenticAMD") != String::npos)
+#if __OGRE_HAVE_NEON == 0
+        if (PlatformInformation::getCpuIdentifier().find("AuthenticAMD") != String::npos)
         {
             // How can I check it's an Athlon XP but not Althon 64?
             // Ok, just test whether supports SSE2/SSE3 or not, if not,
@@ -1056,19 +1057,21 @@ namespace Ogre {
                 mPreferGeneralVersionForSharedBuffers = true;
             }
         }
+#endif
     }
     //---------------------------------------------------------------------
     void OptimisedUtilSSE::softwareVertexSkinning(
         const float *pSrcPos, float *pDestPos,
         const float *pSrcNorm, float *pDestNorm,
         const float *pBlendWeight, const unsigned char* pBlendIndex,
-        const Matrix4* const* blendMatrices,
+        const Affine3* const* blendMatrices,
         size_t srcPosStride, size_t destPosStride,
         size_t srcNormStride, size_t destNormStride,
         size_t blendWeightStride, size_t blendIndexStride,
         size_t numWeightsPerVertex,
         size_t numVertices)
     {
+
         __OGRE_CHECK_STACK_ALIGNED_FOR_SSE();
 
         // All position/normal pointers should be perfect aligned, but still check here
@@ -1197,7 +1200,7 @@ namespace Ogre {
                 }
                 else    // Not 'packed' form or wrong order between position and normal
                 {
-                    // Should never occuring, do nothing here just in case
+                    // Should never occur, do nothing here just in case
                 }
             }
             else    // !pSrcNorm
@@ -1255,7 +1258,7 @@ namespace Ogre {
                 }
                 else    // Not 'packed' form
                 {
-                    // Might occuring only if user forced software blending position only
+                    // Might occur only if user forced software blending position only
                 }
             }
         }
@@ -1282,8 +1285,12 @@ namespace Ogre {
         Real t,
         const float *pSrc1, const float *pSrc2,
         float *pDst,
-        size_t numVertices)
+        size_t pos1VSize, size_t pos2VSize, size_t dstVSize, 
+        size_t numVertices,
+        bool morphNormals)
     {
+        OgreAssert(pos1VSize == pos2VSize && pos2VSize == dstVSize && dstVSize == (morphNormals ? 24 : 12),
+                   "stride not supported");
         __OGRE_CHECK_STACK_ALIGNED_FOR_SSE();
 
         __m128 src01, src02, src11, src12, src21, src22;
@@ -1291,9 +1298,19 @@ namespace Ogre {
 
         __m128 t4 = _mm_load_ps1(&t);
 
-        size_t numIterations = numVertices / 4;
-        numVertices &= 3;
 
+        // If we're morphing normals, we have twice the number of floats to process
+        // Positions are interleaved with normals, so we'll have to separately
+        // normalise just the normals later; we'll just lerp in the first pass
+        // We can't normalise as we go because normals & positions are only 3 floats
+        // each so are not aligned for SSE, we'd mix the data up
+        size_t normalsMultiplier = morphNormals ? 2 : 1;
+        size_t numIterations = (numVertices*normalsMultiplier) / 4;
+        size_t numVerticesRemainder = (numVertices*normalsMultiplier) & 3;
+        
+        // Save for later
+        float *pStartDst = pDst;
+                        
         // Never use meta-function technique to accessing memory because looks like
         // VC7.1 generate a bit inefficient binary code when put following code into
         // inline function.
@@ -1328,7 +1345,7 @@ namespace Ogre {
             }
 
             // Morph remaining vertices
-            switch (numVertices)
+            switch (numVerticesRemainder)
             {
             case 3:
                 // 9 floating-point values
@@ -1352,8 +1369,8 @@ namespace Ogre {
                 // 6 floating-point values
                 src01 = __MM_LOAD_PS(pSrc1 + 0);
                 src02 = __MM_LOAD_PS(pSrc2 + 0);
-                src11 = _mm_loadl_pi(t4, (__m64*)(pSrc1 + 4));  // t4 is meaningless here
-                src12 = _mm_loadl_pi(t4, (__m64*)(pSrc2 + 4));  // t4 is meaningless here
+                src11 = _mm_loadl_pi(t4, (const __m64*)(pSrc1 + 4));  // t4 is meaningless here
+                src12 = _mm_loadl_pi(t4, (const __m64*)(pSrc2 + 4));  // t4 is meaningless here
 
                 dst0 = __MM_LERP_PS(t4, src01, src02);
                 dst1 = __MM_LERP_PS(t4, src11, src12);
@@ -1366,8 +1383,8 @@ namespace Ogre {
                 // 3 floating-point values
                 src01 = _mm_load_ss(pSrc1 + 2);
                 src02 = _mm_load_ss(pSrc2 + 2);
-                src01 = _mm_loadh_pi(src01, (__m64*)(pSrc1 + 0));
-                src02 = _mm_loadh_pi(src02, (__m64*)(pSrc2 + 0));
+                src01 = _mm_loadh_pi(src01, (const __m64*)(pSrc1 + 0));
+                src02 = _mm_loadh_pi(src02, (const __m64*)(pSrc2 + 0));
 
                 dst0 = __MM_LERP_PS(t4, src01, src02);
 
@@ -1376,7 +1393,7 @@ namespace Ogre {
                 break;
             }
         }
-        else    // Should never occuring, just in case buggy driver
+        else    // Should never occur, just in case of buggy drivers
         {
             // Assume all data unaligned
 
@@ -1403,10 +1420,11 @@ namespace Ogre {
                 _mm_storeu_ps(pDst + 4, dst1);
                 _mm_storeu_ps(pDst + 8, dst2);
                 pDst += 12;
+                
             }
 
             // Morph remaining vertices
-            switch (numVertices)
+            switch (numVerticesRemainder)
             {
             case 3:
                 // 9 floating-point values
@@ -1430,8 +1448,8 @@ namespace Ogre {
                 // 6 floating-point values
                 src01 = _mm_loadu_ps(pSrc1 + 0);
                 src02 = _mm_loadu_ps(pSrc2 + 0);
-                src11 = _mm_loadl_pi(t4, (__m64*)(pSrc1 + 4));  // t4 is meaningless here
-                src12 = _mm_loadl_pi(t4, (__m64*)(pSrc2 + 4));  // t4 is meaningless here
+                src11 = _mm_loadl_pi(t4, (const __m64*)(pSrc1 + 4));  // t4 is meaningless here
+                src12 = _mm_loadl_pi(t4, (const __m64*)(pSrc2 + 4));  // t4 is meaningless here
 
                 dst0 = __MM_LERP_PS(t4, src01, src02);
                 dst1 = __MM_LERP_PS(t4, src11, src12);
@@ -1444,8 +1462,8 @@ namespace Ogre {
                 // 3 floating-point values
                 src01 = _mm_load_ss(pSrc1 + 2);
                 src02 = _mm_load_ss(pSrc2 + 2);
-                src01 = _mm_loadh_pi(src01, (__m64*)(pSrc1 + 0));
-                src02 = _mm_loadh_pi(src02, (__m64*)(pSrc2 + 0));
+                src01 = _mm_loadh_pi(src01, (const __m64*)(pSrc1 + 0));
+                src02 = _mm_loadh_pi(src02, (const __m64*)(pSrc2 + 0));
 
                 dst0 = __MM_LERP_PS(t4, src01, src02);
 
@@ -1453,13 +1471,65 @@ namespace Ogre {
                 _mm_store_ss(pDst + 2, dst0);
                 break;
             }
+            
+        }
+        
+        if (morphNormals)
+        {
+            
+            // Now we need to do and unaligned normalise on the normals data we just
+            // lerped; because normals are 3 elements each they're always unaligned
+            float *pNorm = pStartDst;
+            
+            // Offset past first position
+            pNorm += 3;
+            
+            // We'll do one normal each iteration, but still use SSE
+            for (size_t n = 0; n < numVertices; ++n)
+            {
+                // normalise function
+                __m128 norm;
+                
+                // load 3 floating-point normal values
+                // This loads into [0] and clears the rest
+                norm = _mm_load_ss(pNorm + 2);
+                // This loads into [2,3]. [1] is unused
+                norm = _mm_loadh_pi(norm, (__m64*)(pNorm + 0));
+                
+                // Fill a 4-vec with vector length
+                // square
+                __m128 tmp = _mm_mul_ps(norm, norm);
+                // Add - for this we want this effect:
+                // orig   3 | 2 | 1 | 0
+                // add1   0 | 0 | 0 | 2
+                // add2   2 | 3 | 0 | 3
+                // This way elements 0, 2 and 3 have the sum of all entries (except 1 which is unused)
+                
+                tmp = _mm_add_ps(tmp, _mm_shuffle_ps(tmp, tmp, _MM_SHUFFLE(0,0,0,2)));
+                // Add final combination & sqrt 
+                // bottom 3 elements of l will have length, we don't care about 4
+                tmp = _mm_add_ps(tmp, _mm_shuffle_ps(tmp, tmp, _MM_SHUFFLE(2,3,0,3)));
+                // Then divide to normalise
+                norm = _mm_div_ps(norm, _mm_sqrt_ps(tmp));
+                
+                // Store back in the same place
+                _mm_storeh_pi((__m64*)(pNorm + 0), norm);
+                _mm_store_ss(pNorm + 2, norm);
+                
+                // Skip to next vertex (3x normal components, 3x position components)
+                pNorm += 6;
+
+                
+            }
+            
+
         }
     }
     //---------------------------------------------------------------------
     void OptimisedUtilSSE::concatenateAffineMatrices(
-        const Matrix4& baseMatrix,
-        const Matrix4* pSrcMat,
-        Matrix4* pDstMat,
+        const Affine3& baseMatrix,
+        const Affine3* pSrcMat,
+        Affine3* pDstMat,
         size_t numMatrices)
     {
         __OGRE_CHECK_STACK_ALIGNED_FOR_SSE();
@@ -1507,9 +1577,6 @@ namespace Ogre {
             t3 = _mm_mul_ps(m2, m3);    // Compiler should optimise this out of the loop
             __MM_STORE_PS((*pDstMat)[2], __MM_ACCUM4_PS(t0,t1,t2,t3));
 
-            // Row 3
-            __MM_STORE_PS((*pDstMat)[3], m3);
-
             ++pDstMat;
         }
     }
@@ -1525,7 +1592,7 @@ namespace Ogre {
         assert(_isAlignedForSSE(faceNormals));
 
 // Load Vector3 as: (x, 0, y, z)
-#define __LOAD_VECTOR3(p)   _mm_loadh_pi(_mm_load_ss(p), (__m64*)((p)+1))
+#define __LOAD_VECTOR3(p)   _mm_loadh_pi(_mm_load_ss(p), (const __m64*)((p)+1))
 
         // Mask used to changes sign of single precision floating point values.
         OGRE_SIMD_ALIGNED_DECL(static const uint32, msSignMask[4]) =
@@ -1726,8 +1793,13 @@ namespace Ogre {
             bitmask = _mm_movemask_ps(_mm_cmpnle_ps(dp, zero));
 
             // Convert 4-bits mask to 4 bytes, and store results.
+            /*
             *reinterpret_cast<uint32*>(lightFacings) =
                 *reinterpret_cast<const uint32*>(msMaskMapping[bitmask]);
+                */
+            memcpy(lightFacings, msMaskMapping[bitmask], sizeof(uint32));
+            
+            
             lightFacings += 4;
         }
 
@@ -1879,7 +1951,7 @@ namespace Ogre {
             case 2:
                 // 6 floating-point values
                 s0 = SrcAccessor::load(pSrcPos + 0);
-                s1 = _mm_loadl_pi(dir1, (__m64*)(pSrcPos + 4)); // dir1 is meaningless here
+                s1 = _mm_loadl_pi(dir1, (const __m64*)(pSrcPos + 4)); // dir1 is meaningless here
 
                 // The extrusion direction is inverted, use subtract instruction here
                 d0 = _mm_sub_ps(s0, dir0);                      // X0 Y0 Z0 X1
@@ -1891,7 +1963,7 @@ namespace Ogre {
 
             case 1:
                 // 3 floating-point values
-                s0 = _mm_loadl_pi(dir0, (__m64*)(pSrcPos + 0)); // dir0 is meaningless here
+                s0 = _mm_loadl_pi(dir0, (const __m64*)(pSrcPos + 0)); // dir0 is meaningless here
                 s1 = _mm_load_ss(pSrcPos + 2);
 
                 // The extrusion direction is inverted, use subtract instruction here
@@ -1973,7 +2045,7 @@ namespace Ogre {
             for (size_t j = 0; j  < numVertices; ++j)
             {
                 // Load source position
-                __m128 src = _mm_loadh_pi(_mm_load_ss(pSrcPos + 0), (__m64*)(pSrcPos + 1)); // x 0 y z
+                __m128 src = _mm_loadh_pi(_mm_load_ss(pSrcPos + 0), (const __m64*)(pSrcPos + 1)); // x 0 y z
                 pSrcPos += 3;
 
                 // Calculate unnormalised extrusion direction
@@ -2064,6 +2136,7 @@ namespace Ogre {
     //---------------------------------------------------------------------
     //---------------------------------------------------------------------
     //---------------------------------------------------------------------
+    extern OptimisedUtil* _getOptimisedUtilSSE(void);
     extern OptimisedUtil* _getOptimisedUtilSSE(void)
     {
         static OptimisedUtilSSE msOptimisedUtilSSE;
