@@ -4,62 +4,43 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreSceneNode.h"
-
-#include "OgreException.h"
-#include "OgreEntity.h"
-#include "OgreCamera.h"
-#include "OgreLight.h"
-#include "OgreMath.h"
-#include "OgreSceneManager.h"
-#include "OgreMovableObject.h"
-#include "OgreWireBoundingBox.h"
 
 namespace Ogre {
     //-----------------------------------------------------------------------
-    SceneNode::SceneNode(SceneManager* creator)
-        : Node()
-        , mWireBoundingBox(0)
-        , mShowBoundingBox(false)
-        , mCreator(creator)
-        , mYawFixed(false)
-        , mAutoTrackTarget(0)
-        , mIsInSceneGraph(false)
+    SceneNode::SceneNode(SceneManager* creator) : SceneNode(creator, BLANKSTRING)
     {
-        needUpdate();
     }
     //-----------------------------------------------------------------------
     SceneNode::SceneNode(SceneManager* creator, const String& name)
         : Node(name)
-        , mWireBoundingBox(0)
-        , mShowBoundingBox(false)
         , mCreator(creator)
-        , mYawFixed(false)
         , mAutoTrackTarget(0)
+        , mGlobalIndex(-1)
+        , mYawFixed(false)
         , mIsInSceneGraph(false)
+        , mShowBoundingBox(false)
     {
         needUpdate();
     }
@@ -68,18 +49,11 @@ namespace Ogre {
     {
         // Detach all objects, do this manually to avoid needUpdate() call 
         // which can fail because of deleted items
-		ObjectMap::iterator itr;
-		MovableObject* ret;
-		for ( itr = mObjectsByName.begin(); itr != mObjectsByName.end(); itr++ )
-		{
-		  ret = itr->second;
-		  ret->_notifyAttached((SceneNode*)0);
-		}
+        for (ObjectMap::iterator itr = mObjectsByName.begin(); itr != mObjectsByName.end(); ++itr )
+        {
+            (*itr)->_notifyAttached((SceneNode*)0);
+        }
         mObjectsByName.clear();
-
-        if (mWireBoundingBox) {
-			OGRE_DELETE mWireBoundingBox;
-		}
     }
     //-----------------------------------------------------------------------
     void SceneNode::_update(bool updateChildren, bool parentHasChanged)
@@ -88,83 +62,64 @@ namespace Ogre {
         _updateBounds();
     }
     //-----------------------------------------------------------------------
-	void SceneNode::setParent(Node* parent)
-	{
-		Node::setParent(parent);
+    void SceneNode::setParent(Node* parent)
+    {
+        Node::setParent(parent);
 
-		if (parent)
-		{
-			SceneNode* sceneParent = static_cast<SceneNode*>(parent);
-			setInSceneGraph(sceneParent->isInSceneGraph());
-		}
-		else
-		{
-			setInSceneGraph(false);
-		}
-	}
+        if (parent)
+        {
+            SceneNode* sceneParent = static_cast<SceneNode*>(parent);
+            setInSceneGraph(sceneParent->isInSceneGraph());
+        }
+        else
+        {
+            setInSceneGraph(false);
+        }
+    }
     //-----------------------------------------------------------------------
-	void SceneNode::setInSceneGraph(bool inGraph)
-	{
-		if (inGraph != mIsInSceneGraph)
-		{
-			mIsInSceneGraph = inGraph;
-			// Tell children
-	        ChildNodeMap::iterator child;
-    	    for (child = mChildren.begin(); child != mChildren.end(); ++child)
-        	{
-            	SceneNode* sceneChild = static_cast<SceneNode*>(child->second);
-				sceneChild->setInSceneGraph(inGraph);
-			}
-		}
-	}
+    void SceneNode::setInSceneGraph(bool inGraph)
+    {
+        if (inGraph != mIsInSceneGraph)
+        {
+            mIsInSceneGraph = inGraph;
+            // Tell children
+            for (auto child : getChildren())
+            {
+                SceneNode* sceneChild = static_cast<SceneNode*>(child);
+                sceneChild->setInSceneGraph(inGraph);
+            }
+        }
+    }
     //-----------------------------------------------------------------------
+    struct MovableObjectNameExists {
+        const String& name;
+        bool operator()(const MovableObject* mo) {
+            return mo->getName() == name;
+        }
+    };
     void SceneNode::attachObject(MovableObject* obj)
     {
-        if (obj->isAttached())
-        {
-            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
-                "Object already attached to a SceneNode or a Bone",
-                "SceneNode::attachObject");
-        }
+        OgreAssert(!obj->isAttached(), "Object already attached to a SceneNode or a Bone");
 
         obj->_notifyAttached(this);
 
         // Also add to name index
-        std::pair<ObjectMap::iterator, bool> insresult = 
-            mObjectsByName.insert(ObjectMap::value_type(obj->getName(), obj));
-        assert(insresult.second && "Object was not attached because an object of the "
-            "same name was already attached to this node.");
+        MovableObjectNameExists pred = {obj->getName()};
+        ObjectMap::iterator it = std::find_if(mObjectsByName.begin(), mObjectsByName.end(), pred);
+        if (it != mObjectsByName.end())
+            OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM,
+                        "An object named '" + obj->getName() + "' already attached to this SceneNode");
+        mObjectsByName.push_back(obj);
 
         // Make sure bounds get updated (must go right to the top)
         needUpdate();
     }
     //-----------------------------------------------------------------------
-    unsigned short SceneNode::numAttachedObjects(void) const
-    {
-        return static_cast< unsigned short >( mObjectsByName.size() );
-    }
-    //-----------------------------------------------------------------------
-    MovableObject* SceneNode::getAttachedObject(unsigned short index)
-    {
-        if (index < mObjectsByName.size())
-        {
-            ObjectMap::iterator i = mObjectsByName.begin();
-            // Increment (must do this one at a time)            
-            while (index--)++i;
-
-            return i->second;
-        }
-        else
-        {
-            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Object index out of bounds.", "SceneNode::getAttachedObject");
-        }
-        return 0;
-    }
-    //-----------------------------------------------------------------------
-    MovableObject* SceneNode::getAttachedObject(const String& name)
+    MovableObject* SceneNode::getAttachedObject(const String& name) const
     {
         // Look up 
-        ObjectMap::iterator i = mObjectsByName.find(name);
+        MovableObjectNameExists pred = {name};
+        auto i = std::find_if(mObjectsByName.begin(), mObjectsByName.end(), pred);
 
         if (i == mObjectsByName.end())
         {
@@ -172,48 +127,42 @@ namespace Ogre {
                 name + " not found.", "SceneNode::getAttachedObject");
         }
 
-        return i->second;
-
+        return *i;
     }
     //-----------------------------------------------------------------------
     MovableObject* SceneNode::detachObject(unsigned short index)
     {
-        MovableObject* ret;
-        if (index < mObjectsByName.size())
-        {
+        OgreAssert(index < mObjectsByName.size(), "out of bounds");
+        ObjectMap::iterator i = mObjectsByName.begin();
+        i += index;
 
-            ObjectMap::iterator i = mObjectsByName.begin();
-            // Increment (must do this one at a time)            
-            while (index--)++i;
+        MovableObject* ret = *i;
+        std::swap(*i, mObjectsByName.back());
+        mObjectsByName.pop_back();
 
-            ret = i->second;
-            mObjectsByName.erase(i);
-            ret->_notifyAttached((SceneNode*)0);
+        ret->_notifyAttached((SceneNode*)0);
 
-            // Make sure bounds get updated (must go right to the top)
-            needUpdate();
-
-            return ret;
-
-        }
-        else
-        {
-            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Object index out of bounds.", "SceneNode::getAttchedEntity");
-        }
-        return 0;
+        // Make sure bounds get updated (must go right to the top)
+        needUpdate();
 
+        return ret;
     }
     //-----------------------------------------------------------------------
     MovableObject* SceneNode::detachObject(const String& name)
     {
-        ObjectMap::iterator it = mObjectsByName.find(name);
+        MovableObjectNameExists pred = {name};
+        ObjectMap::iterator it = std::find_if(mObjectsByName.begin(), mObjectsByName.end(), pred);
+
         if (it == mObjectsByName.end())
         {
             OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Object " + name + " is not attached "
                 "to this node.", "SceneNode::detachObject");
         }
-        MovableObject* ret = it->second;
-        mObjectsByName.erase(it);
+
+        MovableObject* ret = *it;
+        std::swap(*it, mObjectsByName.back());
+        mObjectsByName.pop_back();
+
         ret->_notifyAttached((SceneNode*)0);
         // Make sure bounds get updated (must go right to the top)
         needUpdate();
@@ -228,9 +177,10 @@ namespace Ogre {
         iend = mObjectsByName.end();
         for (i = mObjectsByName.begin(); i != iend; ++i)
         {
-            if (i->second == obj)
+            if (*i == obj)
             {
-                mObjectsByName.erase(i);
+                std::swap(*i, mObjectsByName.back());
+                mObjectsByName.pop_back();
                 break;
             }
         }
@@ -243,13 +193,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void SceneNode::detachAllObjects(void)
     {
-		ObjectMap::iterator itr;
-		MovableObject* ret;
-		for ( itr = mObjectsByName.begin(); itr != mObjectsByName.end(); itr++ )
-		{
-		  ret = itr->second;
-		  ret->_notifyAttached((SceneNode*)0);
-		}
+        for (ObjectMap::iterator itr = mObjectsByName.begin(); itr != mObjectsByName.end(); ++itr )
+        {
+            (*itr)->_notifyAttached((SceneNode*)0);
+        }
         mObjectsByName.clear();
         // Make sure bounds get updated (must go right to the top)
         needUpdate();
@@ -265,22 +212,21 @@ namespace Ogre {
         for (i = mObjectsByName.begin(); i != mObjectsByName.end(); ++i)
         {
             // Merge world bounds of each object
-            mWorldAABB.merge(i->second->getWorldBoundingBox(true));
+            mWorldAABB.merge((*i)->getWorldBoundingBox(true));
         }
 
         // Merge with children
-        ChildNodeMap::iterator child;
-        for (child = mChildren.begin(); child != mChildren.end(); ++child)
+        for (auto child : getChildren())
         {
-            SceneNode* sceneChild = static_cast<SceneNode*>(child->second);
+            SceneNode* sceneChild = static_cast<SceneNode*>(child);
             mWorldAABB.merge(sceneChild->mWorldAABB);
         }
 
     }
     //-----------------------------------------------------------------------
     void SceneNode::_findVisibleObjects(Camera* cam, RenderQueue* queue, 
-		VisibleObjectsBoundsInfo* visibleBounds, bool includeChildren, 
-		bool displayNodes, bool onlyShadowCasters)
+        VisibleObjectsBoundsInfo* visibleBounds, bool includeChildren, 
+        bool displayNodes, bool onlyShadowCasters)
     {
         // Check self visible
         if (!cam->isVisible(mWorldAABB))
@@ -291,57 +237,33 @@ namespace Ogre {
         ObjectMap::iterator iobjend = mObjectsByName.end();
         for (iobj = mObjectsByName.begin(); iobj != iobjend; ++iobj)
         {
-			MovableObject* mo = iobj->second;
+            MovableObject* mo = *iobj;
 
-			queue->processVisibleObject(mo, cam, onlyShadowCasters, visibleBounds);
+            queue->processVisibleObject(mo, cam, onlyShadowCasters, visibleBounds);
         }
 
         if (includeChildren)
         {
-            ChildNodeMap::iterator child, childend;
-            childend = mChildren.end();
-            for (child = mChildren.begin(); child != childend; ++child)
+            for (auto child : getChildren())
             {
-                SceneNode* sceneChild = static_cast<SceneNode*>(child->second);
+                SceneNode* sceneChild = static_cast<SceneNode*>(child);
                 sceneChild->_findVisibleObjects(cam, queue, visibleBounds, includeChildren, 
-					displayNodes, onlyShadowCasters);
+                    displayNodes, onlyShadowCasters);
             }
         }
 
-        if (displayNodes)
+        if (mCreator && mCreator->getDebugDrawer())
         {
-            // Include self in the render queue
-            queue->addRenderable(this);
+            mCreator->getDebugDrawer()->drawSceneNode(this);
         }
-
-		// Check if the bounding box should be shown.
-		// See if our flag is set or if the scene manager flag is set.
-		if (mShowBoundingBox || (mCreator && mCreator->getShowBoundingBoxes())) 
-		{ 
-			_addBoundingBoxToQueue(queue);
-		}
-
-
     }
 
-
-	void SceneNode::_addBoundingBoxToQueue(RenderQueue* queue) {
-		// Create a WireBoundingBox if needed.
-		if (mWireBoundingBox == NULL) {
-			mWireBoundingBox = OGRE_NEW WireBoundingBox();
-		}
-		mWireBoundingBox->setupBoundingBox(mWorldAABB);
-		queue->addRenderable(mWireBoundingBox);
-	}
-
-	void SceneNode::showBoundingBox(bool bShow) {
-		mShowBoundingBox = bShow;
-	}
-
-	bool SceneNode::getShowBoundingBox() const {
-		return mShowBoundingBox;
-	}
-
+    SceneNode::ObjectIterator SceneNode::getAttachedObjectIterator(void) {
+        return ObjectIterator(mObjectsByName.begin(), mObjectsByName.end());
+    }
+    SceneNode::ConstObjectIterator SceneNode::getAttachedObjectIterator(void) const {
+        return ConstObjectIterator(mObjectsByName.begin(), mObjectsByName.end());
+    }
 
     //-----------------------------------------------------------------------
     void SceneNode::updateFromParentImpl(void) const
@@ -349,11 +271,9 @@ namespace Ogre {
         Node::updateFromParentImpl();
 
         // Notify objects that it has been moved
-        ObjectMap::const_iterator i;
-        for (i = mObjectsByName.begin(); i != mObjectsByName.end(); ++i)
+        for (auto o : mObjectsByName)
         {
-            MovableObject* object = i->second;
-            object->_notifyMoved();
+            o->_notifyMoved();
         }
     }
     //-----------------------------------------------------------------------
@@ -369,21 +289,6 @@ namespace Ogre {
         return mCreator->createSceneNode(name);
     }
     //-----------------------------------------------------------------------
-    const AxisAlignedBox& SceneNode::_getWorldAABB(void) const
-    {
-        return mWorldAABB;
-    }
-    //-----------------------------------------------------------------------
-    SceneNode::ObjectIterator SceneNode::getAttachedObjectIterator(void)
-    {
-        return ObjectIterator(mObjectsByName.begin(), mObjectsByName.end());
-    }
-	//-----------------------------------------------------------------------
-	SceneNode::ConstObjectIterator SceneNode::getAttachedObjectIterator(void) const
-	{
-		return ConstObjectIterator(mObjectsByName.begin(), mObjectsByName.end());
-	}
-    //-----------------------------------------------------------------------
     void SceneNode::removeAndDestroyChild(const String& name)
     {
         SceneNode* pChild = static_cast<SceneNode*>(getChild(name));
@@ -396,43 +301,65 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void SceneNode::removeAndDestroyChild(unsigned short index)
     {
-        SceneNode* pChild = static_cast<SceneNode*>(getChild(index));
+        SceneNode* pChild = static_cast<SceneNode*>(getChildren()[index]);
         pChild->removeAndDestroyAllChildren();
 
         removeChild(index);
-        pChild->getCreator()->destroySceneNode(pChild->getName());
+        pChild->getCreator()->destroySceneNode(pChild);
+    }
+    //-----------------------------------------------------------------------
+    void SceneNode::removeAndDestroyChild(SceneNode* child)
+    {
+        removeAndDestroyChild(std::find(getChildren().begin(), getChildren().end(), child) -
+                              getChildren().begin());
     }
     //-----------------------------------------------------------------------
     void SceneNode::removeAndDestroyAllChildren(void)
     {
-        ChildNodeMap::iterator i, iend;
-        iend = mChildren.end();
-        for (i = mChildren.begin(); i != iend;)
-        {
-            SceneNode* sn = static_cast<SceneNode*>(i->second);
-			// increment iterator before destroying (iterator invalidated by 
-			// SceneManager::destroySceneNode because it causes removal from parent)
-			++i;
+        // do not store iterators (invalidated by
+        // SceneManager::destroySceneNode because it causes removal from parent)
+        while(!getChildren().empty()) {
+            SceneNode* sn = static_cast<SceneNode*>(getChildren().front());
             sn->removeAndDestroyAllChildren();
-            sn->getCreator()->destroySceneNode(sn->getName());
+            sn->getCreator()->destroySceneNode(sn);
         }
-	    mChildren.clear();
+
+        mChildren.clear();
         needUpdate();
     }
+    void SceneNode::loadChildren(const String& filename)
+    {
+        String baseName, strExt;
+        StringUtil::splitBaseFilename(filename, baseName, strExt);
+        auto codec = Codec::getCodec(strExt);
+        if (!codec)
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "No codec found to load " + filename);
+
+        auto stream = Root::openFileStream(
+            filename, ResourceGroupManager::getSingleton().getWorldResourceGroupName());
+        codec->decode(stream, this);
+    }
+    void SceneNode::saveChildren(const String& filename)
+    {
+        String baseName, strExt;
+        StringUtil::splitBaseFilename(filename, baseName, strExt);
+        auto codec = Codec::getCodec(strExt);
+        codec->encodeToFile(this, filename);
+    }
     //-----------------------------------------------------------------------
-	SceneNode* SceneNode::createChildSceneNode(const Vector3& translate, 
-        const Quaternion& rotate)
-	{
-		return static_cast<SceneNode*>(this->createChild(translate, rotate));
-	}
+    SceneNode* SceneNode::createChildSceneNode(const Vector3& inTranslate, 
+        const Quaternion& inRotate)
+    {
+        return static_cast<SceneNode*>(this->createChild(inTranslate, inRotate));
+    }
     //-----------------------------------------------------------------------
-    SceneNode* SceneNode::createChildSceneNode(const String& name, const Vector3& translate, 
-		const Quaternion& rotate)
-	{
-		return static_cast<SceneNode*>(this->createChild(name, translate, rotate));
-	}
+    SceneNode* SceneNode::createChildSceneNode(const String& name, const Vector3& inTranslate, 
+        const Quaternion& inRotate)
+    {
+        return static_cast<SceneNode*>(this->createChild(name, inTranslate, inRotate));
+    }
     //-----------------------------------------------------------------------
-    void SceneNode::findLights(LightList& destList, Real radius) const
+    void SceneNode::findLights(LightList& destList, Real radius, uint32 lightMask) const
     {
         // No any optimisation here, hope inherits more smart for that.
         //
@@ -444,7 +371,7 @@ namespace Ogre {
         if (mCreator)
         {
             // Use SceneManager to calculate
-            mCreator->_populateLightList(this->_getDerivedPosition(), radius, destList);
+            mCreator->_populateLightList(this, radius, destList, lightMask);
         }
         else
         {
@@ -452,7 +379,7 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void SceneNode::setAutoTracking(bool enabled, SceneNode* target, 
+    void SceneNode::setAutoTracking(bool enabled, SceneNode* const target, 
         const Vector3& localDirectionVector,
         const Vector3& offset)
     {
@@ -476,19 +403,19 @@ namespace Ogre {
         mYawFixedAxis = fixedAxis;
     }
 
-	//-----------------------------------------------------------------------
-	void SceneNode::yaw(const Radian& angle, TransformSpace relativeTo)
-	{
-		if (mYawFixed)
-		{
-			rotate(mYawFixedAxis, angle, relativeTo);
-		}
-		else
-		{
-			rotate(Vector3::UNIT_Y, angle, relativeTo);
-		}
+    //-----------------------------------------------------------------------
+    void SceneNode::yaw(const Radian& angle, TransformSpace relativeTo)
+    {
+        if (mYawFixed)
+        {
+            rotate(mYawFixedAxis, angle, relativeTo);
+        }
+        else
+        {
+            rotate(Vector3::UNIT_Y, angle, relativeTo);
+        }
 
-	}
+    }
     //-----------------------------------------------------------------------
     void SceneNode::setDirection(Real x, Real y, Real z, TransformSpace relativeTo, 
         const Vector3& localDirectionVector)
@@ -510,11 +437,11 @@ namespace Ogre {
         switch (relativeTo)
         {
         case TS_PARENT:
-            if (mInheritOrientation)
+            if (getInheritOrientation())
             {
-                if (mParent)
+                if (getParent())
                 {
-                    targetDir = mParent->_getDerivedOrientation() * targetDir;
+                    targetDir = getParent()->_getDerivedOrientation() * targetDir;
                 }
             }
             break;
@@ -531,11 +458,14 @@ namespace Ogre {
         if( mYawFixed )
         {
             // Calculate the quaternion for rotate local Z to target direction
-            Vector3 xVec = mYawFixedAxis.crossProduct(targetDir);
-            xVec.normalise();
-            Vector3 yVec = targetDir.crossProduct(xVec);
-            yVec.normalise();
-            Quaternion unitZToTarget = Quaternion(xVec, yVec, targetDir);
+            Vector3 yawAxis = mYawFixedAxis;
+
+            if (getInheritOrientation() && getParent())
+            {
+                yawAxis = getParent()->_getDerivedOrientation() * yawAxis;
+            }
+
+            Quaternion unitZToTarget = Math::lookRotation(targetDir, yawAxis);
 
             if (localDirectionVector == Vector3::NEGATIVE_UNIT_Z)
             {
@@ -573,8 +503,8 @@ namespace Ogre {
         }
 
         // Set target orientation, transformed to parent space
-        if (mParent && mInheritOrientation)
-            setOrientation(mParent->_getDerivedOrientation().UnitInverse() * targetOrientation);
+        if (getParent() && getInheritOrientation())
+            setOrientation(getParent()->_getDerivedOrientation().UnitInverse() * targetOrientation);
         else
             setOrientation(targetOrientation);
     }
@@ -591,7 +521,7 @@ namespace Ogre {
             origin = _getDerivedPosition();
             break;
         case TS_PARENT:
-            origin = mPosition;
+            origin = getPosition();
             break;
         case TS_LOCAL:
             origin = Vector3::ZERO;
@@ -618,62 +548,50 @@ namespace Ogre {
         return static_cast<SceneNode*>(getParent());
     }
     //-----------------------------------------------------------------------
-    void SceneNode::setVisible(bool visible, bool cascade)
+    void SceneNode::setVisible(bool visible, bool cascade) const
     {
-        ObjectMap::iterator oi, oiend;
-        oiend = mObjectsByName.end();
-        for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
+        for (auto o : mObjectsByName)
         {
-            oi->second->setVisible(visible);
+            o->setVisible(visible);
         }
 
         if (cascade)
         {
-            ChildNodeMap::iterator i, iend;
-            iend = mChildren.end();
-            for (i = mChildren.begin(); i != iend; ++i)
+            for (auto c : getChildren())
             {
-                static_cast<SceneNode*>(i->second)->setVisible(visible, cascade);
+                static_cast<SceneNode*>(c)->setVisible(visible, cascade);
             }
         }
     }
-	//-----------------------------------------------------------------------
-	void SceneNode::setDebugDisplayEnabled(bool enabled, bool cascade)
-	{
-		ObjectMap::iterator oi, oiend;
-		oiend = mObjectsByName.end();
-		for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
-		{
-			oi->second->setDebugDisplayEnabled(enabled);
-		}
-
-		if (cascade)
-		{
-			ChildNodeMap::iterator i, iend;
-			iend = mChildren.end();
-			for (i = mChildren.begin(); i != iend; ++i)
-			{
-				static_cast<SceneNode*>(i->second)->setDebugDisplayEnabled(enabled, cascade);
-			}
-		}
-	}
     //-----------------------------------------------------------------------
-    void SceneNode::flipVisibility(bool cascade)
+    void SceneNode::setDebugDisplayEnabled(bool enabled, bool cascade) const
     {
-        ObjectMap::iterator oi, oiend;
-        oiend = mObjectsByName.end();
-        for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
+        for (auto o : mObjectsByName)
         {
-            oi->second->setVisible(!oi->second->getVisible());
+            o->setDebugDisplayEnabled(enabled);
         }
 
         if (cascade)
         {
-            ChildNodeMap::iterator i, iend;
-            iend = mChildren.end();
-            for (i = mChildren.begin(); i != iend; ++i)
+            for (auto c : getChildren())
             {
-                static_cast<SceneNode*>(i->second)->flipVisibility(cascade);
+                static_cast<SceneNode*>(c)->setDebugDisplayEnabled(enabled, cascade);
+            }
+        }
+    }
+    //-----------------------------------------------------------------------
+    void SceneNode::flipVisibility(bool cascade) const
+    {
+        for (auto o : mObjectsByName)
+        {
+            o->setVisible(!o->getVisible());
+        }
+
+        if (cascade)
+        {
+            for (auto c : getChildren())
+            {
+                static_cast<SceneNode*>(c)->flipVisibility(cascade);
             }
         }
     }
