@@ -819,7 +819,13 @@ void SceneManager::ShadowRenderer::prepareShadowTextures(Camera* cam, Viewport*
             texCam->setLodCamera(cam);
             // set base
             if (light->getType() != Light::LT_POINT)
+            {
+#ifdef OGRE_NODELESS_POSITIONING
                 texCam->getParentSceneNode()->setDirection(light->getDerivedDirection(), Node::TS_WORLD);
+#else
+                texCam->getParentSceneNode()->setOrientation(light->getParentNode()->_getDerivedOrientation());
+#endif
+            }
             if (light->getType() != Light::LT_DIRECTIONAL)
                 texCam->getParentSceneNode()->setPosition(light->getDerivedPosition());
 
@@ -1901,28 +1907,24 @@ void SceneManager::ShadowRenderer::fireShadowTexturesPreReceiver(Light* light, F
         (*i)->shadowTextureReceiverPreViewProj(light, f);
     }
 }
-void SceneManager::ShadowRenderer::sortLightsAffectingFrustum(LightList& lightList)
+void SceneManager::ShadowRenderer::sortLightsAffectingFrustum(LightList& lightList) const
 {
     if ((mShadowTechnique & SHADOWDETAILTYPE_TEXTURE) == 0)
         return;
     // Sort the lights if using texture shadows, since the first 'n' will be
     // used to generate shadow textures and we should pick the most appropriate
 
+    ListenerList listenersCopy = mListeners; // copy in case of listeners removing themselves
+
     // Allow a Listener to override light sorting
     // Reverse iterate so last takes precedence
-    bool overridden = false;
-    ListenerList listenersCopy = mListeners;
-    for (ListenerList::reverse_iterator ri = listenersCopy.rbegin();
-        ri != listenersCopy.rend(); ++ri)
+    for (auto ri = listenersCopy.rbegin(); ri != listenersCopy.rend(); ++ri)
     {
-        overridden = (*ri)->sortLightsAffectingFrustum(lightList);
-        if (overridden)
-            break;
-    }
-    if (!overridden)
-    {
-        // default sort (stable to preserve directional light ordering
-        std::stable_sort(lightList.begin(), lightList.end(), lightsForShadowTextureLess());
+        if((*ri)->sortLightsAffectingFrustum(lightList))
+            return;
     }
+
+    // default sort (stable to preserve directional light ordering
+    std::stable_sort(lightList.begin(), lightList.end(), lightsForShadowTextureLess());
 }
 }
