@@ -29,7 +29,119 @@ THE SOFTWARE.
 #include "OgreVertexIndexData.h"
 #include "OgreHardwareVertexBuffer.h"
 
+#define INT10_MAX ((1 << 9) - 1)
+
 namespace Ogre {
+    static void swapPackedRB(uint32* ptr)
+    {
+        auto cptr = (uint8*)ptr;
+        std::swap(cptr[0], cptr[2]);
+    }
+
+    struct int_10_10_10_2
+    {
+        int32_t x : 10;
+        int32_t y : 10;
+        int32_t z : 10;
+        int32_t w :  2;
+    };
+
+    template<int INCLUDE_W>
+    static void pack_10_10_10_2(uint8* pDst, uint8* pSrc, int elemOffset)
+    {
+        float* pFloat = (float*)(pSrc + elemOffset);
+        int_10_10_10_2 packed = {int(INT10_MAX * pFloat[0]), int(INT10_MAX * pFloat[1]), int(INT10_MAX * pFloat[2]), 1};
+        if(INCLUDE_W)
+            packed.w = int(pFloat[3]);
+        memcpy(pDst + elemOffset, &packed, sizeof(int_10_10_10_2));
+    }
+
+    template<int INCLUDE_W>
+    static void unpack_10_10_10_2(uint8* pDst, uint8* pSrc, int elemOffset)
+    {
+        int_10_10_10_2* pPacked = (int_10_10_10_2*)(pSrc + elemOffset);
+        float* pFloat = (float*)(pDst + elemOffset);
+
+        pFloat[0] = float(pPacked->x) / INT10_MAX;
+        pFloat[1] = float(pPacked->y) / INT10_MAX;
+        pFloat[2] = float(pPacked->z) / INT10_MAX;
+        if(INCLUDE_W)
+            pFloat[3] = pPacked->w;
+    }
+
+    static void copy_float3(uint8* pDst, uint8* pSrc, int elemOffset)
+    {
+        memcpy(pDst, pSrc + elemOffset, sizeof(float) * 3);
+    }
+
+    static void spliceElement(const VertexElement* elem, const HardwareVertexBufferPtr& srcBuf, uint8* pDst,
+                              uint8* pElemDst, uint32 newElemSize, void (*elemConvert)(uint8*, uint8*, int))
+    {
+        auto vertexSize = srcBuf->getVertexSize();
+        auto numVerts = srcBuf->getNumVertices();
+
+        auto elemSize = elem->getSize();
+        auto elemOffset = elem->getOffset();
+
+        auto postVertexOffset = elemOffset + elemSize;
+        auto postVertexSize = vertexSize - postVertexOffset;
+
+        auto elemDstSize = pDst == pElemDst ? newElemSize : 0;
+        size_t newVertexSize = vertexSize - elemSize + elemDstSize;
+        auto elemDstStep = pDst == pElemDst ? newVertexSize : newElemSize;
+
+        HardwareBufferLockGuard srcLock(srcBuf, HardwareBuffer::HBL_READ_ONLY);
+        uint8* pSrc = static_cast<uint8*>(srcLock.pData);
+
+        for (uint32 v = 0; v < numVerts; ++v)
+        {
+            // copy and convert element from vertex
+            elemConvert(pElemDst, pSrc, elemOffset);
+            pElemDst += elemDstStep;
+
+            // copy over other data
+            if (elemOffset)
+                memcpy(pDst, pSrc, elemOffset);
+            if (postVertexSize)
+                memcpy(pDst + elemOffset + elemDstSize, pSrc + postVertexOffset, postVertexSize);
+
+            pSrc += vertexSize;
+            pDst += newVertexSize;
+        }
+    }
+
+    static void updateVertexDeclaration(VertexDeclaration* decl, const VertexElement* elem, VertexElementType newType, uint16 newSource)
+    {
+        auto elemSize = elem->getSize();
+        auto oldElemOffset = elem->getOffset();
+        auto newElemOffset = oldElemOffset;
+
+        auto newElemSize = VertexElement::getTypeSize(newType);
+        auto oldSource = elem->getSource();
+
+        if(newSource != oldSource)
+        {
+            newElemOffset = 0;
+            newElemSize = 0;
+        }
+
+        uint16 idx = 0;
+        for (const auto& e : decl->getElements())
+        {
+            if (&e == elem)
+            {
+                // Modify element
+                decl->modifyElement(idx, newSource, newElemOffset, newType, elem->getSemantic());
+            }
+            else if (e.getSource() == oldSource && e.getOffset() > oldElemOffset)
+            {
+                // shift elements after this one
+                decl->modifyElement(idx, e.getSource(), e.getOffset() - elemSize + newElemSize, e.getType(),
+                                    e.getSemantic(), e.getIndex());
+            }
+            idx++;
+        }
+    }
 
     //-----------------------------------------------------------------------
     VertexData::VertexData(HardwareBufferManagerBase* mgr)
@@ -128,6 +240,56 @@ namespace Ogre {
         
         return dest;
     }
+
+    void VertexData::convertVertexElement(VertexElementSemantic semantic, VertexElementType dstType)
+    {
+        auto elem = vertexDeclaration->findElementBySemantic(semantic);
+
+        if (!elem || VertexElement::getBaseType(elem->getType()) == VertexElement::getBaseType(dstType))
+            return; // nothing to do
+
+        auto srcType = elem->getType();
+        auto vbuf = vertexBufferBinding->getBuffer(elem->getSource());
+
+        size_t newElemSize = VertexElement::getTypeSize(dstType);
+        size_t newVertexSize = vbuf->getVertexSize() - elem->getSize() + newElemSize;
+        auto newVBuf = vbuf->getManager()->createVertexBuffer(newVertexSize, vbuf->getNumVertices(), vbuf->getUsage(),
+                                                              vbuf->hasShadowBuffer());
+
+        {
+            HardwareBufferLockGuard dst(newVBuf, HardwareBuffer::HBL_DISCARD);
+            auto pDst = static_cast<uint8*>(dst.pData);
+
+            if(dstType == VET_INT_10_10_10_2_NORM)
+            {
+                if(srcType == VET_FLOAT3)
+                    spliceElement(elem, vbuf, pDst, pDst, newElemSize, pack_10_10_10_2<false>);
+                else
+                {
+                    OgreAssert(srcType == VET_FLOAT4, "unsupported conversion");
+                    spliceElement(elem, vbuf, pDst, pDst, newElemSize, pack_10_10_10_2<true>);
+                }
+            }
+            else if(dstType == VET_FLOAT3)
+            {
+                OgreAssert(srcType == VET_INT_10_10_10_2_NORM, "unsupported conversion");
+                spliceElement(elem, vbuf, pDst, pDst, newElemSize, unpack_10_10_10_2<false>);
+            }
+            else if(dstType == VET_FLOAT4)
+            {
+                OgreAssert(srcType == VET_INT_10_10_10_2_NORM, "unsupported conversion");
+                spliceElement(elem, vbuf, pDst, pDst, newElemSize, unpack_10_10_10_2<true>);
+            }
+            else
+            {
+                OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "unsupported dstType");
+            }
+        }
+
+        // Bind the new buffer
+        vertexBufferBinding->setBinding(elem->getSource(), newVBuf);
+        updateVertexDeclaration(vertexDeclaration, elem, dstType, elem->getSource());
+    }
     //-----------------------------------------------------------------------
     void VertexData::prepareForShadowVolume(void)
     {
@@ -143,197 +305,106 @@ namespace Ogre {
         when rendering the shadow.
         */
 
-        // Upfront, lets check whether we have vertex program capability
-        RenderSystem* rend = Root::getSingleton().getRenderSystem();
-        bool useVertexPrograms = rend;
-
-
         // Look for a position element
         const VertexElement* posElem = vertexDeclaration->findElementBySemantic(VES_POSITION);
-        if (posElem)
+        if (!posElem)
+            return;
+
+        // Upfront, lets check whether we have vertex program capability
+        bool useVertexPrograms = Root::getSingleton().getRenderSystem() != 0;
+
+        auto vbuf = vertexBufferBinding->getBuffer(posElem->getSource());
+
+        // Are there other elements in the buffer except for the position?
+        // We need to create another buffer to contain the remaining elements
+        // Most drivers don't like gaps in the declaration, and in any case it's waste
+        HardwareVertexBufferPtr newRemainderBuffer;
+        if (vbuf->getVertexSize() > posElem->getSize())
         {
-            size_t v;
-            unsigned short posOldSource = posElem->getSource();
-
-            HardwareVertexBufferSharedPtr vbuf = vertexBufferBinding->getBuffer(posOldSource);
-            bool wasSharedBuffer = false;
-            // Are there other elements in the buffer except for the position?
-            if (vbuf->getVertexSize() > posElem->getSize())
-            {
-                // We need to create another buffer to contain the remaining elements
-                // Most drivers don't like gaps in the declaration, and in any case it's waste
-                wasSharedBuffer = true;
-            }
-            HardwareVertexBufferSharedPtr newPosBuffer, newRemainderBuffer;
-            if (wasSharedBuffer)
-            {
-                newRemainderBuffer = vbuf->getManager()->createVertexBuffer(
-                    vbuf->getVertexSize() - posElem->getSize(), vbuf->getNumVertices(), vbuf->getUsage(),
-                    vbuf->hasShadowBuffer());
-            }
-            // Allocate new position buffer, will be FLOAT3 and 2x the size
-            size_t oldVertexCount = vbuf->getNumVertices();
-            size_t newVertexCount = oldVertexCount * 2;
-            newPosBuffer = vbuf->getManager()->createVertexBuffer(
-                VertexElement::getTypeSize(VET_FLOAT3), newVertexCount, vbuf->getUsage(), 
+            newRemainderBuffer = vbuf->getManager()->createVertexBuffer(
+                vbuf->getVertexSize() - posElem->getSize(), vbuf->getNumVertices(), vbuf->getUsage(),
                 vbuf->hasShadowBuffer());
-
-            // Iterate over the old buffer, copying the appropriate elements and initialising the rest
-            float* pSrc;
-            unsigned char *pBaseSrc = static_cast<unsigned char*>(
-                vbuf->lock(HardwareBuffer::HBL_READ_ONLY));
-            // Point first destination pointer at the start of the new position buffer,
-            // the other one half way along
-            float *pDest = static_cast<float*>(newPosBuffer->lock(HardwareBuffer::HBL_DISCARD));
-            float* pDest2 = pDest + oldVertexCount * 3; 
-
-            // Precalculate any dimensions of vertex areas outside the position
-            size_t prePosVertexSize = 0;
-            unsigned char *pBaseDestRem = 0;
-            if (wasSharedBuffer)
-            {
-                size_t postPosVertexSize, postPosVertexOffset;
-                pBaseDestRem = static_cast<unsigned char*>(
-                    newRemainderBuffer->lock(HardwareBuffer::HBL_DISCARD));
-                prePosVertexSize = posElem->getOffset();
-                postPosVertexOffset = prePosVertexSize + posElem->getSize();
-                postPosVertexSize = vbuf->getVertexSize() - postPosVertexOffset;
-                // the 2 separate bits together should be the same size as the remainder buffer vertex
-                assert (newRemainderBuffer->getVertexSize() == prePosVertexSize + postPosVertexSize);
-
-                // Iterate over the vertices
-                for (v = 0; v < oldVertexCount; ++v)
-                {
-                    // Copy position, into both buffers
-                    posElem->baseVertexPointerToElement(pBaseSrc, &pSrc);
-                    *pDest++ = *pDest2++ = *pSrc++;
-                    *pDest++ = *pDest2++ = *pSrc++;
-                    *pDest++ = *pDest2++ = *pSrc++;
-
-                    // now deal with any other elements 
-                    // Basically we just memcpy the vertex excluding the position
-                    if (prePosVertexSize > 0)
-                        memcpy(pBaseDestRem, pBaseSrc, prePosVertexSize);
-                    if (postPosVertexSize > 0)
-                        memcpy(pBaseDestRem + prePosVertexSize, 
-                            pBaseSrc + postPosVertexOffset, postPosVertexSize);
-                    pBaseDestRem += newRemainderBuffer->getVertexSize();
-
-                    pBaseSrc += vbuf->getVertexSize();
-
-                } // next vertex
-            }
-            else
-            {
-                // Unshared buffer, can block copy the whole thing
-                memcpy(pDest, pBaseSrc, vbuf->getSizeInBytes());
-                memcpy(pDest2, pBaseSrc, vbuf->getSizeInBytes());
-            }
-
-            vbuf->unlock();
-            newPosBuffer->unlock();
-            if (wasSharedBuffer)
-                newRemainderBuffer->unlock();
-
-            // At this stage, he original vertex buffer is going to be destroyed
-            // So we should force the deallocation of any temporary copies
-            vbuf->getManager()->_forceReleaseBufferCopies(vbuf);
-
-            if (useVertexPrograms)
-            {
-                // Now it's time to set up the w buffer
-                hardwareShadowVolWBuffer = vbuf->getManager()->createVertexBuffer(
-                    sizeof(float), newVertexCount, HardwareBuffer::HBU_STATIC_WRITE_ONLY, false);
-                // Fill the first half with 1.0, second half with 0.0
-                pDest = static_cast<float*>(
-                    hardwareShadowVolWBuffer->lock(HardwareBuffer::HBL_DISCARD));
-                for (v = 0; v < oldVertexCount; ++v)
-                {
-                    *pDest++ = 1.0f;
-                }
-                for (v = 0; v < oldVertexCount; ++v)
-                {
-                    *pDest++ = 0.0f;
-                }
-                hardwareShadowVolWBuffer->unlock();
-            }
-
-            unsigned short newPosBufferSource; 
-            if (wasSharedBuffer)
-            {
-                // Get the a new buffer binding index
-                newPosBufferSource= vertexBufferBinding->getNextIndex();
-                // Re-bind the old index to the remainder buffer
-                vertexBufferBinding->setBinding(posOldSource, newRemainderBuffer);
-            }
-            else
-            {
-                // We can just re-use the same source idex for the new position buffer
-                newPosBufferSource = posOldSource;
-            }
-            // Bind the new position buffer
-            vertexBufferBinding->setBinding(newPosBufferSource, newPosBuffer);
-
-            // Now, alter the vertex declaration to change the position source
-            // and the offsets of elements using the same buffer
-            VertexDeclaration::VertexElementList::const_iterator elemi = 
-                vertexDeclaration->getElements().begin();
-            VertexDeclaration::VertexElementList::const_iterator elemiend = 
-                vertexDeclaration->getElements().end();
-            unsigned short idx;
-            for(idx = 0; elemi != elemiend; ++elemi, ++idx) 
-            {
-                if (&(*elemi) == posElem)
-                {
-                    // Modify position to point at new position buffer
-                    vertexDeclaration->modifyElement(
-                        idx, 
-                        newPosBufferSource, // new source buffer
-                        0, // no offset now
-                        VET_FLOAT3, 
-                        VES_POSITION);
-                }
-                else if (wasSharedBuffer &&
-                    elemi->getSource() == posOldSource &&
-                    elemi->getOffset() > prePosVertexSize )
-                {
-                    // This element came after position, remove the position's
-                    // size
-                    vertexDeclaration->modifyElement(
-                        idx, 
-                        posOldSource, // same old source
-                        elemi->getOffset() - posElem->getSize(), // less offset now
-                        elemi->getType(), 
-                        elemi->getSemantic(),
-                        elemi->getIndex());
-
-                }
-
-            }
-
-
-            // Note that we don't change vertexCount, because the other buffer(s) are still the same
-            // size after all
-
-
         }
+        // Allocate new position buffer, will be FLOAT3 and 2x the size
+        size_t oldVertexCount = vbuf->getNumVertices();
+        size_t newVertexCount = oldVertexCount * 2;
+        auto newPosBuffer = vbuf->getManager()->createVertexBuffer(
+            VertexElement::getTypeSize(VET_FLOAT3), newVertexCount, vbuf->getUsage(), vbuf->hasShadowBuffer());
+
+        // Point first destination pointer at the start of the new position buffer,
+        // the other one half way along
+        auto pDest = static_cast<float*>(newPosBuffer->lock(HardwareBuffer::HBL_DISCARD));
+        auto pDest2 = pDest + oldVertexCount * 3;
+
+        if (newRemainderBuffer)
+        {
+            // Basically we just memcpy the vertex excluding the position
+            HardwareBufferLockGuard destRemLock(newRemainderBuffer, HardwareBuffer::HBL_DISCARD);
+            spliceElement(posElem, vbuf, (uint8*)destRemLock.pData, (uint8*)pDest, posElem->getSize(), copy_float3);
+        }
+        else
+        {
+            // Unshared buffer, can block copy the whole thing
+            vbuf->readData(0, vbuf->getSizeInBytes(), pDest);
+        }
+
+        memcpy(pDest2, pDest, oldVertexCount * 3 * sizeof(float));
+
+        newPosBuffer->unlock();
+
+        // At this stage, he original vertex buffer is going to be destroyed
+        // So we should force the deallocation of any temporary copies
+        vbuf->getManager()->_forceReleaseBufferCopies(vbuf);
+
+        if (useVertexPrograms)
+        {
+            // Now it's time to set up the w buffer
+            hardwareShadowVolWBuffer =
+                vbuf->getManager()->createVertexBuffer(sizeof(float), newVertexCount, HBU_GPU_ONLY, false);
+            // Fill the first half with 1.0, second half with 0.0
+            pDest = static_cast<float*>(hardwareShadowVolWBuffer->lock(HardwareBuffer::HBL_DISCARD));
+            for (size_t v = 0; v < oldVertexCount; ++v)
+            {
+                *pDest++ = 1.0f;
+            }
+            // Fill the second half with 0.0
+            memset(pDest, 0, sizeof(float) * oldVertexCount);
+            hardwareShadowVolWBuffer->unlock();
+        }
+
+        auto newPosBufferSource = posElem->getSource();
+        if (newRemainderBuffer)
+        {
+            // Get the a new buffer binding index
+            newPosBufferSource= vertexBufferBinding->getNextIndex();
+            // Re-bind the old index to the remainder buffer
+            vertexBufferBinding->setBinding(posElem->getSource(), newRemainderBuffer);
+        }
+
+        // Bind the new position buffer
+        vertexBufferBinding->setBinding(newPosBufferSource, newPosBuffer);
+
+        // Now, alter the vertex declaration to change the position source
+        // and the offsets of elements using the same buffer
+        // Note that we don't change vertexCount, because the other buffer(s) are still the same
+        // size after all
+        updateVertexDeclaration(vertexDeclaration, posElem, VET_FLOAT3, newPosBufferSource);
     }
     //-----------------------------------------------------------------------
-    void VertexData::reorganiseBuffers(VertexDeclaration* newDeclaration, 
-        const BufferUsageList& bufferUsages, HardwareBufferManagerBase* mgr)
+    void VertexData::reorganiseBuffers(VertexDeclaration* newDeclaration, const BufferUsageList& bufferUsages,
+                                       HardwareBufferManagerBase* mgr)
     {
         HardwareBufferManagerBase* pManager = mgr ? mgr : mMgr;
         // Firstly, close up any gaps in the buffer sources which might have arisen
         newDeclaration->closeGapsInSource();
 
         // Build up a list of both old and new elements in each buffer
-        unsigned short buf = 0;
-        std::vector<void*> oldBufferLocks;
+        std::vector<uint8*> oldBufferLocks;
         std::vector<size_t> oldBufferVertexSizes;
-        std::vector<void*> newBufferLocks;
+        std::vector<uint8*> newBufferLocks;
         std::vector<size_t> newBufferVertexSizes;
         VertexBufferBinding* newBinding = pManager->createVertexBufferBinding();
-        const VertexBufferBinding::VertexBufferBindingMap& oldBindingMap = vertexBufferBinding->getBindings();
+        const auto& oldBindingMap = vertexBufferBinding->getBindings();
         VertexBufferBinding::VertexBufferBindingMap::const_iterator itBinding;
 
         // Pre-allocate old buffer locks
@@ -347,90 +418,64 @@ namespace Ogre {
         bool useShadowBuffer = false;
 
         // Lock all the old buffers for reading
-        for (itBinding = oldBindingMap.begin(); itBinding != oldBindingMap.end(); ++itBinding)
+        for (const auto& it : oldBindingMap)
         {
-            assert(itBinding->second->getNumVertices() >= vertexCount);
+            assert(it.second->getNumVertices() >= vertexCount);
 
-            oldBufferVertexSizes[itBinding->first] =
-                itBinding->second->getVertexSize();
-            oldBufferLocks[itBinding->first] =
-                itBinding->second->lock(
-                    HardwareBuffer::HBL_READ_ONLY);
+            oldBufferVertexSizes[it.first] = it.second->getVertexSize();
+            oldBufferLocks[it.first] = (uint8*)it.second->lock(HardwareBuffer::HBL_READ_ONLY);
 
-            useShadowBuffer |= itBinding->second->hasShadowBuffer();
+            useShadowBuffer |= it.second->hasShadowBuffer();
         }
         
         // Create new buffers and lock all for writing
-        buf = 0;
+        uint16 buf = 0;
         while (!newDeclaration->findElementsBySource(buf).empty())
         {
             size_t vertexSize = newDeclaration->getVertexSize(buf);
 
-            HardwareVertexBufferSharedPtr vbuf = 
-                pManager->createVertexBuffer(
-                    vertexSize,
-                    vertexCount, 
-                    bufferUsages[buf], useShadowBuffer);
+            auto vbuf = pManager->createVertexBuffer(vertexSize, vertexCount, bufferUsages[buf], useShadowBuffer);
             newBinding->setBinding(buf, vbuf);
 
             newBufferVertexSizes.push_back(vertexSize);
-            newBufferLocks.push_back(
-                vbuf->lock(HardwareBuffer::HBL_DISCARD));
+            newBufferLocks.push_back((uint8*)vbuf->lock(HardwareBuffer::HBL_DISCARD));
             buf++;
         }
 
         // Map from new to old elements
-        typedef std::map<const VertexElement*, const VertexElement*> NewToOldElementMap;
-        NewToOldElementMap newToOldElementMap;
-        const VertexDeclaration::VertexElementList& newElemList = newDeclaration->getElements();
-        VertexDeclaration::VertexElementList::const_iterator ei, eiend;
-        eiend = newElemList.end();
-        for (ei = newElemList.begin(); ei != eiend; ++ei)
+        std::map<const VertexElement*, const VertexElement*>  newToOldElementMap;
+        const auto& newElemList = newDeclaration->getElements();
+        for (const auto& newElem : newElemList)
         {
             // Find corresponding old element
-            const VertexElement* oldElem = 
-                vertexDeclaration->findElementBySemantic(
-                    (*ei).getSemantic(), (*ei).getIndex());
+            auto oldElem = vertexDeclaration->findElementBySemantic(newElem.getSemantic(), newElem.getIndex());
             if (!oldElem)
             {
                 // Error, cannot create new elements with this method
-                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
-                    "Element not found in old vertex declaration", 
-                    "VertexData::reorganiseBuffers");
+                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Element not found in old vertex declaration");
             }
-            newToOldElementMap[&(*ei)] = oldElem;
+            newToOldElementMap[&newElem] = oldElem;
         }
         // Now iterate over the new buffers, pulling data out of the old ones
         // For each vertex
         for (size_t v = 0; v < vertexCount; ++v)
         {
             // For each (new) element
-            for (ei = newElemList.begin(); ei != eiend; ++ei)
+            for (const auto& newElem : newElemList)
             {
-                const VertexElement* newElem = &(*ei);
-                NewToOldElementMap::iterator noi = newToOldElementMap.find(newElem);
-                const VertexElement* oldElem = noi->second;
-                unsigned short oldBufferNo = oldElem->getSource();
-                unsigned short newBufferNo = newElem->getSource();
-                void* pSrcBase = static_cast<void*>(
-                    static_cast<unsigned char*>(oldBufferLocks[oldBufferNo])
-                    + v * oldBufferVertexSizes[oldBufferNo]);
-                void* pDstBase = static_cast<void*>(
-                    static_cast<unsigned char*>(newBufferLocks[newBufferNo])
-                    + v * newBufferVertexSizes[newBufferNo]);
-                void *pSrc, *pDst;
-                oldElem->baseVertexPointerToElement(pSrcBase, &pSrc);
-                newElem->baseVertexPointerToElement(pDstBase, &pDst);
-                
-                memcpy(pDst, pSrc, newElem->getSize());
-                
+                const VertexElement* oldElem = newToOldElementMap[&newElem];
+                auto oldBufferNo = oldElem->getSource();
+                auto newBufferNo = newElem.getSource();
+                auto pSrc = oldBufferLocks[oldBufferNo] + v * oldBufferVertexSizes[oldBufferNo];
+                auto pDst = newBufferLocks[newBufferNo] + v * newBufferVertexSizes[newBufferNo];
+                memcpy(pDst + newElem.getOffset(), pSrc + oldElem->getOffset(), newElem.getSize());
             }
         }
 
         // Unlock all buffers
-        for (itBinding = oldBindingMap.begin(); itBinding != oldBindingMap.end(); ++itBinding)
+        for (const auto& it : oldBindingMap)
         {
-            itBinding->second->unlock();
+            it.second->unlock();
         }
         for (buf = 0; buf < newBinding->getBufferCount(); ++buf)
         {
@@ -600,7 +645,7 @@ namespace Ogre {
                         {
                             uint32* pRGBA;
                             elem.baseVertexPointerToElement(pBase, &pRGBA);
-                            VertexElement::convertColourValue(_DETAIL_SWAP_RB, destType, pRGBA);
+                            swapPackedRB(pRGBA);
                         }
                     }
                     pBase = static_cast<void*>(
@@ -660,6 +705,43 @@ namespace Ogre {
         
         return supportedCount;
     }
+    VertexData* VertexData::_cloneRemovingBlendData() const
+    {
+        // Clone without copying data
+        VertexData* ret = clone(false);
+        bool removeIndices = Root::getSingleton().isBlendIndicesGpuRedundant();
+        bool removeWeights = Root::getSingleton().isBlendWeightsGpuRedundant();
+
+        unsigned short safeSource = 0xFFFF;
+        auto blendIndexElem = vertexDeclaration->findElementBySemantic(VES_BLEND_INDICES);
+        if (blendIndexElem && removeIndices)
+        {
+            //save the source in order to prevent the next stage from unbinding it.
+            safeSource = blendIndexElem->getSource();
+            // Remove buffer reference
+            ret->vertexBufferBinding->unsetBinding(blendIndexElem->getSource());
+        }
+
+        // Remove blend weights
+        const VertexElement* blendWeightElem = vertexDeclaration->findElementBySemantic(VES_BLEND_WEIGHTS);
+        if (removeWeights && blendWeightElem && blendWeightElem->getSource() != safeSource)
+        {
+            // Remove buffer reference
+            ret->vertexBufferBinding->unsetBinding(blendWeightElem->getSource());
+        }
+
+        // remove elements from declaration
+        if (removeIndices)
+            ret->vertexDeclaration->removeElement(VES_BLEND_INDICES);
+        if (removeWeights)
+            ret->vertexDeclaration->removeElement(VES_BLEND_WEIGHTS);
+
+        // Close gaps in bindings for effective and safely
+        if (removeWeights || removeIndices)
+            ret->closeGapsInBindings();
+
+        return ret;
+    }
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     IndexData::IndexData()
