@@ -40,7 +40,7 @@ namespace Ogre {
     *  @{
     */
     /** Specialisation of HighLevelGpuProgram to provide support for nVidia's CG language.
-    @remarks
+
         Cg can be used to compile common, high-level, C-like code down to assembler
         language for both GL and Direct3D, for multiple graphics cards. You must
         supply a list of profiles which your program must support using
@@ -51,13 +51,6 @@ namespace Ogre {
     class CgProgram : public HighLevelGpuProgram
     {
     public:
-        /// Command object for setting entry point
-        class CmdEntryPoint : public ParamCommand
-        {
-        public:
-            String doGet(const void* target) const;
-            void doSet(void* target, const String& val);
-        };
         /// Command object for setting profiles
         class CmdProfiles : public ParamCommand
         {
@@ -75,7 +68,6 @@ namespace Ogre {
 
     protected:
 
-        static CmdEntryPoint msCmdEntryPoint;
         static CmdProfiles msCmdProfiles;
         static CmdArgs msCmdArgs;
 
@@ -101,7 +93,6 @@ namespace Ogre {
         void mapTypeAndElementSize(CGtype cgType, bool isRegisterCombiner, GpuConstantDefinition& def) const;
 
         StringVector mProfiles;
-        String mEntryPoint;
         String mSelectedProfile;
         String mProgramString;
         CGprofile mSelectedCgProfile;
