@@ -107,8 +107,6 @@ namespace Ogre
             const Frustum *frustum;
             /// texture 
             IDirect3DBaseTexture9 *pTex;
-            /// vertex texture 
-            IDirect3DBaseTexture9 *pVertexTex;
         } mTexStageDesc[OGRE_MAX_TEXTURE_LAYERS];
 
         // Array of up to 8 lights, indexed as per API
@@ -279,8 +277,6 @@ namespace Ogre
         void _setPointParameters(bool attenuationEnabled, Real minSize, Real maxSize);
         void _setTexture(size_t unit, bool enabled, const TexturePtr& texPtr);
         void _setSampler(size_t unit, Sampler& sampler);
-        void _setVertexTexture(size_t unit, const TexturePtr& tex);
-        void _disableTextureUnit(size_t texUnit);
         void _setTextureCoordSet( size_t unit, size_t index );
         void _setTextureCoordCalculation(size_t unit, TexCoordCalcMethod m, 
             const Frustum* frustum = 0);
@@ -386,10 +382,7 @@ namespace Ogre
         void createStereoDriver(const NameValuePairList* miscParams);
 #endif
 
-    protected:  
-        /// Returns the sampler id for a given unit texture number
-        DWORD getSamplerId(size_t unit);
-
+    protected:
         /// Notify when a device has been lost.
         void notifyOnDeviceLost(D3D9Device* device);
 
