@@ -69,7 +69,7 @@ namespace Ogre  {
             {GL_NONE},                                           // PF_UNKNOWN
             {GL_RED, GL_UNSIGNED_BYTE, GL_R8},                   // PF_L8
             {GL_RED, GL_UNSIGNED_SHORT, GL_R16},                 // PF_L16
-            {GL_ALPHA, GL_UNSIGNED_BYTE, GL_R8},                 // PF_A8
+            {GL_RED, GL_UNSIGNED_BYTE, GL_R8},                   // PF_A8
             {GL_RG, GL_UNSIGNED_BYTE, GL_RG8},                   // PF_BYTE_LA
             {GL_RGB, GL_UNSIGNED_SHORT_5_6_5, GL_RGB5},          // PF_R5G6B5
             {GL_BGR, GL_UNSIGNED_SHORT_5_6_5, GL_RGB5},          // PF_B5G6R5
@@ -179,6 +179,8 @@ namespace Ogre  {
             {GL_NONE, GL_NONE, GL_COMPRESSED_RGBA_ASTC_12x10_KHR},// PF_ASTC_RGBA_12X10_LDR
             {GL_NONE, GL_NONE, GL_COMPRESSED_RGBA_ASTC_12x12_KHR},// PF_ASTC_RGBA_12X12_LDR
             {GL_DEPTH_COMPONENT, GL_UNSIGNED_INT, GL_DEPTH_COMPONENT32}, // PF_DEPTH32
+            {GL_DEPTH_COMPONENT, GL_FLOAT, GL_DEPTH_COMPONENT32F}, // PF_DEPTH32F
+            {GL_DEPTH_COMPONENT, GL_UNSIGNED_INT, GL_DEPTH24_STENCIL8}, // PF_DEPTH24_STENCIL8
     };
 
     GLenum GL3PlusPixelUtil::getGLOriginFormat(PixelFormat pf)
@@ -336,11 +338,12 @@ namespace Ogre  {
         switch(format)
         {
         case GL_DEPTH_COMPONENT24:
-        case GL_DEPTH_COMPONENT32F:
         case GL_DEPTH_COMPONENT:
             return PF_DEPTH16;
         case GL_DEPTH_COMPONENT32:
             return PF_DEPTH32;
+        case GL_DEPTH_COMPONENT32F:
+            return PF_DEPTH32F;
         case GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT:
             return PF_DXT1;
         case GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT:
