@@ -23,13 +23,10 @@ class _OgreSampleClassExport Sample_ParticleGS : public SdkSample
     void testCapabilities(const RenderSystemCapabilities* caps);
     void setupContent(void);
     void cleanupContent();
-    bool frameStarted(const FrameEvent& evt);
 #ifdef LOG_GENERATED_BUFFER
     bool frameEnded(const FrameEvent& evt);
 #endif
-    Real demoTime;
     ProceduralManualObject* mParticleSystem;
-    ProceduralManualObjectFactory *mProceduralManualObjectFactory;
 };
 }
 #endif
