@@ -96,6 +96,7 @@ PlaypenTestPlugin::PlaypenTestPlugin()
     addSample(new PlayPen_NonUniqueResourceNames());
     addSample(new PlayPen_NormalMapMirroredUVs()); // texture is missing
     addSample(new PlayPen_Ortho());
+    addSample(new PlayPen_PBR());
     addSample(new PlayPen_PointSprites()); // not sure if RTSS (via gl_PointSize shader input) or glPointSize that makes points too small
     addSample(new PlayPen_PoseAnimationWithNormals()); // bugged on Lin
     addSample(new PlayPen_PoseAnimationWithoutNormals()); // bugged on Lin
@@ -130,6 +131,7 @@ PlaypenTestPlugin::PlaypenTestPlugin()
     addSample(new PlayPen_TextureShadowsTransparentCaster()); // crashes on Windows
 
     addSample(new PlayPen_TransparencyMipMaps());
+    addSample(new PlayPen_TwoSidedLighting());
     addSample(new PlayPen_VertexTexture()); // needs to be implemented in OpenGL
     addSample(new PlayPen_ViewportNoShadows());
     addSample(new PlayPen_WindowedViewportMode()); // possibly should only show quarter of screen?
