@@ -27,13 +27,10 @@ THE SOFTWARE.
 */
 
 #include "Ogre.h"
-#include "OgreMeshSerializer.h"
-#include "OgreSkeletonSerializer.h"
 #include "OgreDefaultHardwareBufferManager.h"
 #include "OgreMeshLodGenerator.h"
 #include "OgreDistanceLodStrategy.h"
 #include "OgreLodStrategyManager.h"
-#include "OgreHardwareVertexBuffer.h"
 #include "OgrePixelCountLodStrategy.h"
 #include "OgreLodConfig.h"
 
@@ -47,34 +44,35 @@ namespace {
 
 void help(void)
 {
-    // Print help message
-    cout << endl << "OgreMeshUpgrader: Upgrades or downgrades .mesh file versions." << endl;
-    cout << "Provided for OGRE by Steve Streeting 2004-2014" << endl << endl;
-    cout << "Usage: OgreMeshUpgrader [opts] sourcefile [destfile] " << endl;
-    cout << "-i             = Interactive mode, prompt for options" << endl;
-    cout << "-autogen       = Generate autoconfigured LOD. No more LOD options needed!" << endl;
-    cout << "-l lodlevels   = number of LOD levels" << endl;
-    cout << "-d loddist     = distance increment to reduce LOD" << endl;
-    cout << "-p lodpercent  = Percentage triangle reduction amount per LOD" << endl;
-    cout << "-f lodnumtris  = Fixed vertex reduction per LOD" << endl;
-    cout << "-e         = DON'T generate edge lists (for stencil shadows)" << endl;
-    cout << "-t         = Generate tangents (for normal mapping)" << endl;
-    cout << "-td [uvw|tangent]" << endl;
-    cout << "           = Tangent vertex semantic destination (default tangent)" << endl;
-    cout << "-ts [3|4]      = Tangent size (3 or 4 components, 4 includes parity, default 3)" << endl;
-    cout << "-tm            = Split tangent vertices at UV mirror points" << endl;
-    cout << "-tr            = Split tangent vertices where basis is rotated > 90 degrees" << endl;
-    cout << "-r         = DON'T reorganise buffers to recommended format" << endl;
-    cout << "-E endian  = Set endian mode 'big' 'little' or 'native' (default)" << endl;
-    cout << "-b         = Recalculate bounding box (static meshes only)" << endl;
-    cout << "-V version = Specify OGRE version format to write instead of latest" << endl;
-    cout << "             Options are: 1.10, 1.8, 1.7, 1.4, 1.0" << endl;
-    cout << "-log filename  = name of the log file (default: 'OgreMeshUpgrader.log')" << endl;
-    cout << "sourcefile = name of file to convert" << endl;
-    cout << "destfile   = optional name of file to write to. If you don't" << endl;
-    cout << "             specify this OGRE overwrites the existing file." << endl;
+    cout <<
+R"HELP(Usage: OgreMeshUpgrader [opts] sourcefile [destfile]
 
-    cout << endl;
+  Upgrades or downgrades .mesh file versions.
+
+-i             = Interactive mode, prompt for options
+-pack          = Pack normals and tangents as int_10_10_10_2
+-autogen       = Generate autoconfigured LOD. No LOD options needed
+-l lodlevels   = number of LOD levels
+-d loddist     = distance increment to reduce LOD
+-p lodpercent  = Percentage triangle reduction amount per LOD
+-f lodnumtris  = Fixed vertex reduction per LOD
+-e             = DON'T generate edge lists (for stencil shadows)
+-t             = Generate tangents (for normal mapping)
+-td [uvw|tangent]
+               = Tangent vertex semantic destination (default: tangent)
+-ts [3|4]      = Tangent size (4 includes parity, default: 3)
+-tm            = Split tangent vertices at UV mirror points
+-tr            = Split tangent vertices where basis is rotated > 90 degrees
+-r             = DON'T reorganise buffers to recommended format
+-E endian      = Set endian mode 'big' 'little' or 'native' (default)
+-b             = Recalculate bounding box (static meshes only)
+-V version     = Specify OGRE version format to write instead of latest
+                 Options are: 1.10, 1.8, 1.7, 1.4, 1.0
+-log filename  = name of the log file (default: 'OgreMeshUpgrader.log')
+sourcefile     = name of file to convert
+destfile       = optional name of file to write to. If you don't
+                 specify this OGRE overwrites the existing file.
+)HELP";
 }
 
 struct UpgradeOptions {
@@ -87,6 +85,7 @@ struct UpgradeOptions {
     bool tangentSplitRotated;
     bool dontReorganise;
     bool lodAutoconfigure;
+    bool packNormalsTangents;
     unsigned short numLods;
     Real lodDist;
     Real lodPercent;
@@ -98,23 +97,10 @@ struct UpgradeOptions {
     String logFile;
 };
 
-// Crappy globals
-// NB some of these are not directly used, but are required to
-//   instantiate the singletons used in the dlls
-LogManager* logMgr = 0;
-Math* mth = 0;
-LodStrategyManager* lodMgr = 0;
-MaterialManager* matMgr = 0;
-SkeletonManager* skelMgr = 0;
-MeshSerializer* meshSerializer = 0;
-SkeletonSerializer* skeletonSerializer = 0;
-DefaultHardwareBufferManager *bufferManager = 0;
-ResourceGroupManager* rgm = 0;
-MeshManager* meshMgr = 0;
-UpgradeOptions opts;
-
-void parseOpts(UnaryOptionList& unOpts, BinaryOptionList& binOpts)
+UpgradeOptions parseOpts(UnaryOptionList& unOpts, BinaryOptionList& binOpts)
 {
+    UpgradeOptions opts;
+
     // Defaults
     opts.interactive = false;
     opts.suppressEdgeLists = false;
@@ -126,6 +112,8 @@ void parseOpts(UnaryOptionList& unOpts, BinaryOptionList& binOpts)
     opts.dontReorganise = false;
     opts.endian = Serializer::ENDIAN_NATIVE;
 
+    opts.packNormalsTangents = false;
+
     opts.lodAutoconfigure = false;
     opts.lodDist = 500;
     opts.lodFixed = 0;
@@ -142,6 +130,7 @@ void parseOpts(UnaryOptionList& unOpts, BinaryOptionList& binOpts)
     opts.lodAutoconfigure = unOpts["-autogen"];
     opts.interactive = unOpts["-i"];
     opts.dontReorganise = unOpts["-r"];
+    opts.packNormalsTangents = unOpts["-pack"];
 
     // Unary options (true/false options that don't take a parameter)
     if (unOpts["-b"]) {
@@ -216,9 +205,11 @@ void parseOpts(UnaryOptionList& unOpts, BinaryOptionList& binOpts)
         } else if (bi->second == "1.0") {
             opts.targetVersion = MESH_VERSION_1_0;
         } else {
-            logMgr->logError("Unrecognised target mesh version '" + bi->second + "'");
+            LogManager::getSingleton().logError("Unrecognised target mesh version '" + bi->second + "'");
         }
     }
+
+    return opts;
 }
 
 String describeSemantic(VertexElementSemantic sem)
@@ -431,7 +422,7 @@ void reorganiseVertexBuffers(const String& desc, Mesh& mesh, SubMesh* sm, Vertex
 }
 
 // Utility function to allow the user to modify the layout of vertex buffers.
-void reorganiseVertexBuffers(Mesh& mesh)
+void reorganiseVertexBuffers(const UpgradeOptions& opts, Mesh& mesh)
 {
     // Make sure animation types up to date
     mesh._determineAnimationTypes();
@@ -483,7 +474,7 @@ void reorganiseVertexBuffers(Mesh& mesh)
     }
 }
 
-void vertexBufferReorg(Mesh& mesh)
+void vertexBufferReorg(const UpgradeOptions& opts, Mesh& mesh)
 {
     String response;
 
@@ -495,7 +486,7 @@ void vertexBufferReorg(Mesh& mesh)
             cin >> response;
             StringUtil::toLowerCase(response);
             if (response == "y") {
-                reorganiseVertexBuffers(mesh);
+                reorganiseVertexBuffers(opts, mesh);
             } else if (response == "n") {
                 // Do nothing
             } else {
@@ -504,7 +495,7 @@ void vertexBufferReorg(Mesh& mesh)
             }
         }
     } else if (!opts.dontReorganise) {
-                reorganiseVertexBuffers(mesh);
+                reorganiseVertexBuffers(opts, mesh);
             }
 }
 
@@ -552,6 +543,7 @@ void recalcBounds(Mesh* mesh)
 
 void printLodConfig(const LodConfig& lodConfig)
 {
+    auto logMgr = LogManager::getSingletonPtr();
     logMgr->logMessage("LOD config summary:");
     logMgr->logMessage(" - lodConfig.strategy=" + lodConfig.strategy->getName());
     String reductionMethod("Unknown");
@@ -592,7 +584,7 @@ size_t getUniqueVertexCount(MeshPtr mesh)
     return lodConfig.levels[0].outUniqueVertexCount;
 }
 
-void buildLod(MeshPtr& mesh)
+void buildLod(UpgradeOptions& opts, MeshPtr& mesh)
 {
     String response;
 
@@ -819,9 +811,9 @@ void buildLod(MeshPtr& mesh)
     }
     printLodConfig(lodConfig);
 
-    logMgr->logMessage("Generating LOD levels...");
+    LogManager::getSingleton().logMessage("Generating LOD levels...");
     gen.generateLodLevels(lodConfig);
-    logMgr->logMessage("Generating LOD levels... success");
+    LogManager::getSingleton().logMessage("Generating LOD levels... success");
 }
 
 void checkColour(VertexData* vdata, bool& hasColour, bool& hasAmbiguousColour,
@@ -910,13 +902,13 @@ int main(int numargs, char** args)
     }
 
     int retCode = 0;
+
+    LogManager logMgr;
+    // this log catches output from the parseArgs call and routes it to stdout only
+    logMgr.createLog("Temporary log", true, true, true);
+
     try
     {
-        logMgr = new LogManager();
-        //logMgr->createLog("OgreMeshUpgrader.log", true);
-        // this log catches output from the parseArgs call and routes it to stdout only
-        logMgr->createLog("Temporary log", false, true, true);
-
         UnaryOptionList unOptList;
         BinaryOptionList binOptList;
 
@@ -928,6 +920,7 @@ int main(int numargs, char** args)
         unOptList["-r"] = false;
         unOptList["-byte"] = true; // this is the only option now, dont error if specified
         unOptList["-autogen"] = false;
+        unOptList["-pack"] = false;
         unOptList["-b"] = false;
         binOptList["-l"] = "";
         binOptList["-d"] = "";
@@ -940,30 +933,26 @@ int main(int numargs, char** args)
         binOptList["-log"] = "OgreMeshUpgrader.log";
 
         int startIdx = findCommandLineOpts(numargs, args, unOptList, binOptList);
-        parseOpts(unOptList, binOptList);
+        auto opts = parseOpts(unOptList, binOptList);
+
+        logMgr.setDefaultLog(NULL); // swallow startup messages
+        Root root("", "", "");
+        // get rid of the temporary log as we use the new log now
+        logMgr.destroyLog("Temporary log");
 
         // use the log specified by the cmdline params
-        logMgr->setDefaultLog(logMgr->createLog(opts.logFile, true, true));
-
-        // get rid of the temporary log as we use the new log now
-        logMgr->destroyLog("Temporary log");
+        logMgr.setDefaultLog(logMgr.createLog(opts.logFile, true, true));
 
         String source(args[startIdx]);
 
-        rgm = new ResourceGroupManager();
-        mth = new Math();
-        lodMgr = new LodStrategyManager();
-        matMgr = new MaterialManager();
-        matMgr->initialise();
-        skelMgr = new SkeletonManager();
-        meshSerializer = new MeshSerializer();
+        MaterialManager::getSingleton().initialise();
+        MeshSerializer meshSerializer;
         MeshResourceCreator resCreator;
-        meshSerializer->setListener(&resCreator);
-        skeletonSerializer = new SkeletonSerializer();
-        bufferManager = new DefaultHardwareBufferManager(); // needed because we don't have a rendersystem
-        meshMgr = new MeshManager();
+        meshSerializer.setListener(&resCreator);
+        SkeletonSerializer skeletonSerializer;
+        DefaultHardwareBufferManager bufferManager; // needed because we don't have a rendersystem
         // don't pad during upgrade
-        meshMgr->setBoundsPaddingFactor(0.0f);
+        MeshManager::getSingleton().setBoundsPaddingFactor(0.0f);
 
         // Load the mesh
         struct stat tagStat;
@@ -981,12 +970,11 @@ int main(int numargs, char** args)
                 "Unexpected error while reading file " + source, "OgreMeshUpgrader");
         fclose( pFile );
 
-        MeshPtr meshPtr = MeshManager::getSingleton().createManual("conversion",
-                                                                   ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
+        MeshPtr meshPtr = MeshManager::getSingleton().createManual("conversion", RGN_DEFAULT);
         Mesh* mesh = meshPtr.get();
 
         DataStreamPtr stream(memstream);
-        meshSerializer->importMesh(stream, mesh);
+        meshSerializer.importMesh(stream, mesh);
 
         // Write out the converted mesh
         String dest;
@@ -998,12 +986,12 @@ int main(int numargs, char** args)
 
         String response;
 
-        vertexBufferReorg(*mesh);
+        vertexBufferReorg(opts, *mesh);
 
         // Deal with VET_COLOUR ambiguities
         resolveColourAmbiguities(mesh);
 
-        buildLod(meshPtr);
+        buildLod(opts, meshPtr);
 
         if (opts.interactive) {
             do {
@@ -1026,9 +1014,9 @@ int main(int numargs, char** args)
         } else {
         // Make sure we generate edge lists, provided they are not deliberately disabled
             if (!opts.suppressEdgeLists) {
-                logMgr->logMessage("Generating edge lists...");
+                logMgr.logMessage("Generating edge lists...");
                 mesh->buildEdgeList();
-                logMgr->logMessage("Generating edge lists... success");
+                logMgr.logMessage("Generating edge lists... success");
             } else {
                 mesh->freeEdgeList();
 			}
@@ -1077,35 +1065,33 @@ int main(int numargs, char** args)
 
             }
             if (opts.generateTangents) {
-                logMgr->logMessage("Generating tangent vectors...");
+                logMgr.logMessage("Generating tangent vectors...");
                 mesh->buildTangentVectors(opts.tangentSemantic, srcTex, destTex,
                     opts.tangentSplitMirrored, opts.tangentSplitRotated,
                     opts.tangentUseParity);
-                logMgr->logMessage("Generating tangent vectors... success");
+                logMgr.logMessage("Generating tangent vectors... success");
             }
         }
 
+        if(opts.packNormalsTangents)
+        {
+            mesh->_convertVertexElement(VES_NORMAL, VET_INT_10_10_10_2_NORM);
+            mesh->_convertVertexElement(VES_TANGENT, VET_INT_10_10_10_2_NORM);
+        }
+
         if (opts.recalcBounds) {
             recalcBounds(mesh);
         }
 
-        meshSerializer->exportMesh(mesh, dest, opts.targetVersion, opts.endian);
+        meshSerializer.exportMesh(mesh, dest, opts.targetVersion, opts.endian);
+
+        logMgr.setDefaultLog(NULL); // swallow shutdown messages
     }
     catch (Exception& e)
     {
-        cout << " !!! Exception caught: " << e.getDescription() << endl;
+        LogManager::getSingleton().logError(e.getDescription());
         retCode = 1;
     }
 
-    delete meshMgr;
-    delete skeletonSerializer;
-    delete meshSerializer;
-    delete skelMgr;
-    delete matMgr;
-    delete lodMgr;
-    delete mth;
-    delete rgm;
-    delete logMgr;
-
     return retCode;
 }
